import mock
import unittest

from pythonwarrior.abilities.detonate import Detonate
from pythonwarrior.floor import Floor
from pythonwarrior.units.base import UnitBase
from pythonwarrior.units.warrior import Warrior


class TestDetonate(unittest.TestCase):
    def setUp(self):
        self.floor = Floor()
        self.floor.width = 4
        self.floor.height = 3
        self.warrior = Warrior()
        self.warrior._health = 20
        self.detonate = Detonate(self.warrior)

        self.target = UnitBase()
        self.target._health = 10

        self.floor.add(self.warrior, 0, 1, 'east')

    def test_should_deal_8_damage_if_forward_1_space(self):
        self.floor.add(self.target, 1, 1)
        self.detonate.perform()
        self.assertEqual(2, self.target._health)

    def test_should_deal_4_damage_if_forward_2_spaces(self):
        self.floor.add(self.target, 2, 1)
        self.detonate.perform()
        self.assertEqual(6, self.target._health)

    def test_should_deal_4_damage_if_diagonal_1_space(self):
        self.floor.add(self.target, 1, 2)
        self.detonate.perform()
        self.assertEqual(6, self.target._health)

        self.floor.add(self.target, 1, 0)
        self.detonate.perform()
        self.assertEqual(2, self.target._health)

    def test_should_deal_4_damage_to_the_user(self):
        self.detonate.perform()
        self.assertEqual(16, self.warrior._health)

    def test_should_cause_ticking_captives_to_explode(self):
        mock_explode = mock.Mock()
        self.target.abilities['explode_'] = mock_explode
        self.floor.add(self.target, 1, 1)
        self.detonate.perform()
        self.assertTrue(mock_explode.perform.called)



import base64
import os
import pickle
import re

import warriorpyCore.src.config.Config as Config
import warriorpyCore.src.level.Level as Level
import warriorpyCore.src.tower.Tower as Tower


class Profile(object):
    def __init__(self):
        self.epic = False
        self.tower_path = None
        self.warrior_name = None
        self.score = 0
        self.epic_score = 0
        self.current_epic_score = 0
        self.current_epic_grades = {}
        self.average_grade = None
        self.abilities = []
        self.level_number = 0
        self.last_level_number = None
        self._player_path = None

    def encode(self):
        return base64.b64encode(pickle.dumps(self))

    def save(self):
        self.update_epic_score()
        if self.epic:
            self.level_number = 0
        f = open(self.player_path + '/.profile', 'w')
        f.write(self.encode())

    @staticmethod
    def decode(encoded_profile):
        return pickle.loads(base64.b64decode(encoded_profile))

    @staticmethod
    def load(path):
        f = open(path)
        player = Profile.decode(f.read())
        player._player_path = os.path.dirname(path)
        return player

    @property
    def player_path(self):
        if self._player_path is None:
            self._player_path = Config.path_prefix + \
                "/warriorpy/%s" % self.directory_name()
        return self._player_path

    def directory_name(self):
        return "-".join([re.sub("[^a-z0-9]", "-", self.warrior_name.lower()),
                        self.tower().name()])

    def __repr__(self):
        return " - ".join([self.warrior_name, self.tower().name(),
                           "level %s" % self.level_number,
                           "score %s" % self.score])

    def tower(self):
        return Tower(os.path.basename(self.tower_path))

    def current_level(self):
        return Level(self, self.level_number)

    def next_level(self):
        return Level(self, self.level_number+1)

    def add_abilities(self, *abilities):
        self.abilities += list(set(abilities))

    def enable_epic_mode(self):
        self.epic = True
        if not hasattr(self, 'epic_score'):
            self.epic_score = 0
        if not hasattr(self, 'current_epic_score'):
            self.current_epic_score = 0
        self.current_epic_score = self.current_epic_score or 0
        if not hasattr(self, 'last_level_number'):
            self.last_level_number = self.level_number
        self.last_level_number = self.last_level_number or self.level_number

    def enable_normal_mode(self):
        self.epic = False
        self.epic_score = 0
        self.current_epic_score = 0
        self.current_epic_grades = {}
        self.average_grade = None
        self.level_number = self.last_level_number
        self.last_level_number = None

    def level_after_epic(self):
        if self.last_level_number:
            return Level(self, self.last_level_number + 1).exists()

    def update_epic_score(self):
        if self.current_epic_score > self.epic_score:
            self.epic_score = self.current_epic_score
            self.average_grade = self.calculate_average_grade()

    def calculate_average_grade(self):
        if len(self.current_epic_grades) > 0:
            summed_values = sum(self.current_epic_grades.values())
            return summed_values / len(self.current_epic_grades)



import os
import unittest
from pythonwarrior.player_generator import PlayerGenerator
from pythonwarrior.level import Level
from pythonwarrior.profile import Profile


class TestPlayerGenerator(unittest.TestCase):
    def setUp(self):
        self.level = Level(Profile(), 15)
        self.previous_level = Level(Profile(), 14)
        self.generator = PlayerGenerator(self.level, self.previous_level)

    def test_should_know_templates_path(self):
        expected = os.path.normpath(os.path.dirname(__file__) +
                                    '/../templates'),
        self.assertEqual(self.generator.templates_path(), expected[0])



import unittest

from pythonwarrior.abilities.listen import Listen
from pythonwarrior.floor import Floor
from pythonwarrior.units.base import UnitBase
from pythonwarrior.units.warrior import Warrior


class TestListen(unittest.TestCase):
    def setUp(self):
        self.floor = Floor()
        self.floor.width = 2
        self.floor.height = 3
        self.warrior = Warrior()
        self.floor.add(self.warrior, 0, 0)
        self.listen = Listen(self.warrior)

    def test_should_return_space_with_units_but_not_main_unit(self):
        self.floor.add(UnitBase(), 0, 1)
        self.assertEqual(1, len(self.listen.perform()))



import unittest

from pythonwarrior.units.thick_sludge import ThickSludge


class TestThickSludge(unittest.TestCase):
    def setUp(self):
        self.sludge = ThickSludge()

    def test_should_have_24_max_health(self):
        self.assertEqual(24, self.sludge.max_health)

    def test_should_appear_as_S_on_map(self):
        self.assertEqual("S", self.sludge.character)

    def test_should_have_the_name_Thick_Sludge(self):
        self.assertEqual("ThickSludge", self.sludge.name())



import mock
import unittest

from pythonwarrior.abilities.rest import Rest
from pythonwarrior.units.warrior import Warrior


class TestRest(unittest.TestCase):
    def setUp(self):
        self.warrior = Warrior()
        self.rest = Rest(self.warrior)

    def test_should_give_10_percent_health_back(self):
        self.warrior.max_health = 20
        self.warrior._health = 10
        self.rest.perform()
        self.assertEqual(12, self.warrior.health)

    def test_should_not_add_health_when_at_max(self):
        self.warrior.max_health = 20
        self._health = 20
        self.rest.perform()
        self.assertEqual(20, self.warrior.health)

    def test_should_not_go_over_max_health(self):
        self.warrior.max_health = 20
        self._health = 19
        self.rest.perform()
        self.assertEqual(20, self.warrior.health)

    def test_should_say_something_if_at_full_health(self):
        self.warrior.max_health = 20
        self.warrior.say = mock.Mock()
        self._health = 20
        self.rest.perform()
        self.assertTrue(self.warrior.say.called)



#  --------
# |C @ S aa|
#  --------

level.description("The wall behind you feels a bit further away in this room. And you hear more cries for help.")
level.tip("You can walk backward by passing 'backward' as an argument to warrior.walk_. Same goes for warrior.feel, warrior.rescue_ and warrior.attack_. Archers have a limited attack distance.")
level.clue("Walk backward if you are taking damage from afar and do not have enough health to attack. You may also want to consider walking backward until warrior.feel('backward').is_wall().")

level.time_bonus(55)
level.ace_score(105)
level.size(8, 1)
level.stairs(7, 0)

level.warrior(2, 0, 'east')

level.unit('captive', 0, 0, 'east')
level.unit('thick_sludge', 4, 0, 'west')
level.unit('archer', 6, 0, 'west')
level.unit('archer', 7, 0, 'west')



#  ----
# |C s |
# | @ S|
# |C s>|
#  ----

level.description("Your ears become more in tune with the surroundings. "
                  "Listen to find enemies and captives!")
level.tip("Use warrior.listen to find spaces with other units, "
          "and warrior.direction_of to determine what direction they're in.")
level.clue("Walk towards an enemy or captive with "
           "warrior.walk_(warrior.direction_of(warrior.listen()[0])), "
           "once len(warrior.listen()) == 0 then head for the stairs.")
level.time_bonus(55)
level.ace_score(144)
level.size(4, 3)
level.stairs(3, 2)

def add_abilities(warrior):
    warrior.add_abilities('listen')
    warrior.add_abilities('direction_of')

level.warrior(1, 1, 'east', func=add_abilities)

level.unit('captive', 0, 0, 'east')
level.unit('captive', 0, 2, 'east')
level.unit('sludge', 2, 0, 'south')
level.unit('thick_sludge', 3, 1, 'west')
level.unit('sludge', 2, 2, 'north')



import unittest
from pythonwarrior.tower import Tower


class TestTower(unittest.TestCase):
    def setUp(self):
        self.tower = Tower('path/to/tower')

    def test_should_consider_last_part_of_path_as_name(self):
        self.assertEqual(self.tower.name(), 'tower')

    def test_should_use_name_when_converting_to_string(self):
        self.assertEqual(str(self.tower), self.tower.name())



import unittest
from pythonwarrior.floor import Floor
from pythonwarrior.units.warrior import Warrior
from pythonwarrior.units.sludge import Sludge
from pythonwarrior.units.captive import Captive
from pythonwarrior.units.golem import Golem


class TestSpace(unittest.TestCase):
    def setUp(self):
        self.floor = Floor()
        self.floor.width = 2
        self.floor.height = 3


class TestEmptySpace(TestSpace):
    def setUp(self):
        super(TestEmptySpace, self).setUp()
        self.space = self.floor.space(0, 0)

    def test_should_not_be_enemy(self):
        self.assertFalse(self.space.is_enemy())

    def test_should_not_be_warrior(self):
        self.assertFalse(self.space.is_warrior())

    def test_should_be_empty(self):
        self.assertTrue(self.space.is_empty())

    def test_should_not_be_stairs(self):
        self.assertFalse(self.space.is_stairs())

    def test_should_not_be_captive(self):
        self.assertFalse(self.space.is_captive())

    def test_should_say_nothing_as_name(self):
        self.assertEqual(str(self.space), 'nothing')


class TestOutOfBounds(TestSpace):
    def setUp(self):
        super(TestOutOfBounds, self).setUp()
        self.space = self.floor.space(-1, 1)

    def test_should_be_wall(self):
        self.assertTrue(self.space.is_wall())

    def test_should_not_be_empty(self):
        self.assertFalse(self.space.is_empty())

    def test_should_have_name_of_wall(self):
        self.assertEqual(str(self.space), 'wall')


class TestWithWarrior(TestSpace):
    def setUp(self):
        super(TestWithWarrior, self).setUp()
        warrior = Warrior()
        self.floor.add(warrior, 0, 0)
        self.space = self.floor.space(0, 0)

    def test_should_be_warrior(self):
        self.assertTrue(self.space.is_warrior())

    def test_should_be_player(self):
        self.assertTrue(self.space.is_player())

    def test_should_not_be_enemy(self):
        self.assertFalse(self.space.is_enemy())

    def test_should_not_be_empty(self):
        self.assertFalse(self.space.is_empty())

    def test_should_know_what_unit_is_on_that_space(self):
        self.assertEqual(self.space.unit.__class__.__name__, "Warrior")


class TestWithEnemy(TestSpace):
    def setUp(self):
        super(TestWithEnemy, self).setUp()
        sludge = Sludge()
        self.floor.add(sludge, 0, 0)
        self.space = self.floor.space(0, 0)

    def test_should_be_enemy(self):
        self.assertTrue(self.space.is_enemy())

    def test_should_not_be_warrior(self):
        self.assertFalse(self.space.is_warrior())

    def test_should_not_be_empty(self):
        self.assertFalse(self.space.is_empty())

    def test_should_have_name_of_unit(self):
        self.assertEqual(str(self.space), 'Sludge')

    def test_bound_enemy_should_be_captive(self):
        self.space.unit.bind()
        self.assertTrue(self.space.is_captive)

    def test_bound_enemy_should_not_look_like_enemy(self):
        self.space.unit.bind()
        self.assertFalse(self.space.is_enemy())


class TestWithCaptive(TestSpace):
    def setUp(self):
        super(TestWithCaptive, self).setUp()
        self.captive = Captive()
        self.floor.add(self.captive, 0, 0)
        self.space = self.floor.space(0, 0)

    def test_should_be_captive(self):
        self.assertTrue(self.space.is_captive())

    def test_should_not_be_enemy(self):
        self.assertFalse(self.space.is_enemy())

    def test_should_be_ticking_if_captive_has_time_bomb(self):
        self.captive.add_abilities('explode_')
        self.assertTrue(self.space.is_ticking())

    def test_should_not_be_ticking_if_captive_does_not_have_time_bomb(self):
        self.assertFalse(self.space.is_ticking())


class TestWithGolem(TestSpace):
    def setUp(self):
        super(TestWithGolem, self).setUp()
        self.golem = Golem()
        self.floor.add(self.golem, 0, 0)
        self.space = self.floor.space(0, 0)

    def test_should_be_golem(self):
        self.assertTrue(self.space.is_golem())

    def test_should_not_be_enemy(self):
        self.assertFalse(self.space.is_enemy())

    def test_should_be_player(self):
        self.assertTrue(self.space.is_player())


class TestAtStairs(TestSpace):
    def setUp(self):
        super(TestAtStairs, self).setUp()
        self.floor.place_stairs(0, 0)
        self.space = self.floor.space(0, 0)

    def test_should_be_empty(self):
        self.assertTrue(self.space.is_empty())

    def test_should_be_stairs(self):
        self.assertTrue(self.space.is_stairs())


if __name__ == '__main__':
    unittest.main()



#  -------
# |@  Cww>|
#  -------

level.description("You hear the mumbling of wizards. Beware of their deadly wands! Good thing you found a bow.")
level.tip("Use warrior.look() to determine your surroundings, and warrior.shoot_() to fire an arrow.")
level.clue("Wizards are deadly but low in health. Kill them before they have time to attack.")

level.time_bonus(20)
level.ace_score(46)
level.size(6, 1)
level.stairs(5, 0)

def add_abilities(warrior):
    warrior.add_abilities('look', 'shoot_')
level.warrior(0, 0, 'east', func=add_abilities)

level.unit('captive', 2, 0, 'west')
level.unit('wizard', 3, 0, 'west')
level.unit('wizard', 4, 0, 'west')



#  --------
# |@   s  >|
#  --------

level.description("It is too dark to see anything, but you smell sludge nearby.")
level.tip("Use warrior.feel().is_empty() to see if there is anything in front of you, and warrior.attack() to fight it. Remember, you can only do one action (ending in _) per turn.")
level.time_bonus(20)
level.ace_score(26)
level.size(8, 1)
level.stairs(7, 0)
def add_abilities(warrior):
    warrior.add_abilities('feel', 'attack_')
level.warrior(0, 0, 'east', func=add_abilities)
level.unit('sludge', 4, 0, 'west')



import os


class Tower(object):
    def __init__(self, path):
        self.path = os.path.join(os.path.abspath('towers'), path)

    def name(self):
        return os.path.basename(self.path)

    def __repr__(self):
        return self.name()



#  -----
# | sC >|
# |@ s C|
# | s   |
#  -----

level.description("Another ticking sound, but some sludge is blocking the way.")
level.tip("Quickly kill the sludge and rescue the captive before the "
          "bomb goes off. You can't simply go around them.")
level.clue("Determine the direction of the ticking captive and kill any "
           "enemies blocking that path. You may need to bind surrounding "
           "enemies first.")
level.time_bonus(70)
level.ace_score(134)
level.size(5, 3)
level.stairs(4, 0)

level.warrior(0, 1, 'east')

level.unit('sludge', 1, 0, 'south')
level.unit('sludge', 1, 2, 'north')
level.unit('sludge', 2, 1, 'west')
level.unit('captive', 2, 0, 'west')

def add_abilities(unit):
    unit.add_abilities('explode_')
    unit.abilities_attr['explode_'].time = 10

level.unit('captive', 4, 1, 'west', func=add_abilities)




from absoluteDirections import ABSOLUTE_DIRECTIONS
from relativeDirections import RELATIVE_DIRECTIONS
from verifyAbsoluteDirection import verifyAbsoluteDirection
from verifyRelativeDirection import verifyRelativeDirection

# /**
#  * Returns the absolute direction for a given direction, with reference to
#  * another direction (reference direction).
#  *
#  * @param {string} direction The direction (relative).
#  * @param {string} referenceDirection The reference direction (absolute).
#  *
#  * @returns {string} The absolute direction.
#  */

def getAbsoluteDirection(direction, referenceDirection):
    verifyRelativeDirection(direction)
    verifyAbsoluteDirection(referenceDirection)

    index = (ABSOLUTE_DIRECTIONS.index(referenceDirection) + RELATIVE_DIRECTIONS.index(direction)) % 4

    return ABSOLUTE_DIRECTIONS[index]



from .base import AbilityBase


class DirectionOfStairs(AbilityBase):
    def description(self):
        return ("Returns the direction ('left', 'right', 'forward',"
                "'backward') the stairs are from your location")

    def perform(self):
        return self._unit.position.relative_direction_of_stairs()



import mock
import unittest

from pythonwarrior.config import Config
from pythonwarrior.ui import UI
from io import BytesIO


class TestUI(unittest.TestCase):
    def setUp(self):
        self.ui = UI()
        self.config = Config
        self.config.delay = None
        self.out_stream = BytesIO()
        self.in_stream = BytesIO()
        self.config.out_stream = self.out_stream
        self.config.in_stream = self.in_stream

    def test_should_add_puts_to_out_stream(self):
        self.ui.puts("hello")
        self.assertEqual(self.out_stream.getvalue(), "hello\n")

    def test_should_add_write_to_out_stream_without_newline(self):
        self.ui.write("hello")
        self.out_stream.seek(0)
        self.assertEqual(self.out_stream.getvalue(), "hello")

    def test_should_fetch_gets_from_in_stream(self):
        self.in_stream.write("bar")
        self.in_stream.seek(0)
        self.assertEqual(self.ui.gets(), "bar")

    def test_should_gets_should_return_empty_string_if_no_input(self):
        self.config.in_stream = None
        self.assertEqual(self.ui.gets(), "")

    def test_should_request_text_input(self):
        self.in_stream.write("bar")
        self.in_stream.seek(0)
        self.assertEqual(self.ui.request("foo"), "bar")
        self.assertEqual(self.out_stream.getvalue(), "foo")

    @mock.patch('pythonwarrior.ui.UI.request')
    def test_should_ask_for_yes_no_and_return_true_when_yes(self, mock_req):
        mock_req.return_value = 'y'
        self.assertTrue(self.ui.ask("foo?"))

    @mock.patch('pythonwarrior.ui.UI.request')
    def test_should_ask_for_yes_no_and_return_false_when_no(self, mock_req):
        mock_req.return_value = 'n'
        self.assertFalse(self.ui.ask("foo?"))

    @mock.patch('pythonwarrior.ui.UI.request')
    def test_should_ask_for_yes_no_and_return_false_for_any_input(self,
                                                                  mock_req):
        mock_req.return_value = 'aklhasdf'
        self.assertFalse(self.ui.ask("foo?"))

    @mock.patch('pythonwarrior.ui.UI.request')
    def test_should_present_multiple_options_and_return_selected_one(self,
                                                                     mock_req):
        mock_req.return_value = '2'
        self.assertEqual(self.ui.choose('item', ['foo', 'bar', 'test']), 'bar')
        output = self.out_stream.getvalue()
        assert '[1] foo' in output
        assert '[2] bar' in output
        assert '[3] test' in output

    @mock.patch('pythonwarrior.ui.UI.request')
    def test_choose_should_accept_array_as_option(self, mock_req):
        mock_req.return_value = 3
        self.assertEqual(
            self.ui.choose('item', ['foo', 'bar', ['tower', 'easy']]),
            'tower'
        )
        assert '[3] easy' in self.out_stream.getvalue()

    @mock.patch('pythonwarrior.ui.UI.puts')
    @mock.patch('pythonwarrior.ui.UI.gets')
    @mock.patch('pythonwarrior.ui.UI.request')
    def test_choose_should_return_option_without_prompt_if_only_one_item(
            self, mock_req, mock_gets, mock_puts):
        self.assertEqual(self.ui.choose('item', ['foo']), 'foo')
        assert not mock_puts.called
        assert not mock_gets.called

    def test_choose_should_return_first_value_in_array_if_one_item(self):
        self.assertEqual(self.ui.choose('item', [['foo', 'bar']]), 'foo')

    @mock.patch('time.sleep')
    @mock.patch('pythonwarrior.ui.UI.puts')
    def test_should_delay_after_puts_when_specified(self,
                                                    mock_puts,
                                                    mock_sleep):
        self.config.delay = 1.3
        self.ui.puts_with_delay('foo')
        mock_puts.assert_called_with('foo')
        mock_sleep.assert_called_with(1.3)

    @mock.patch('time.sleep')
    @mock.patch('pythonwarrior.ui.UI.puts')
    def test_should_not_delay_puts_when_no_delay(self,
                                                 mock_puts,
                                                 mock_sleep):
        self.ui.puts_with_delay('foo')
        mock_puts.assert_called_with('foo')
        assert not mock_sleep.called



from relativeDirections import RELATIVE_DIRECTIONS

# /**
#  * Checks if the given direction is a valid relative direction.
#  *
#  * @param {string} direction The direction.
#  *
#  * @throws Will throw if the direction is not valid.
#  */

def verifyRelativeDirection(direction):
        if direction not in RELATIVE_DIRECTIONS:
            raise Exception("Unknown direction %s. "
                            "Should be 'forward', 'backward', "
                            "'left' or 'right'")


#  ----
# |@s  |
# | sS>|
#  ----

level.description("Another large room, but with several enemies "
                  "blocking your way to the stairs.")
level.tip("Just like walking, you can attack_ and feel in multiple "
          "directions ('forward', 'left', 'right', 'backward').")
level.clue("Call warrior.feel(direction).is_enemy() in each direction "
           "to make sure there isn't an enemy beside you "
           "(attack if there is). "
           "Call warrior.rest_ if you're low and health when there "
           "are no enemies around.")
level.time_bonus(40)
level.ace_score(84)
level.size(4, 2)
level.stairs(3, 1)

def add_abilities(warrior):
    warrior.add_abilities('attack_')
    warrior.add_abilities('health')
    warrior.add_abilities('rest_')

level.warrior(0, 0, 'east', func=add_abilities)

level.unit('sludge', 1, 0, 'west')
level.unit('thick_sludge', 2, 1, 'west')
level.unit('sludge', 1, 1, 'north')



# /**
#  * Returns the Manhattan distance of a location from another location (reference
#  * location).
#  *
#  * @param {number[]} location The location as [x, y].
#  * @param {number[]} referenceLocation The reference location as [x, y].
#  *
#  * @returns {number} The distance between the locations.
#  */

def getDistanceOfLocation(x1, y1, x2, y2):
    return abs(x2 - x1) + abs(y2 - y1)


import unittest
from pythonwarrior.units.base import UnitBase
from pythonwarrior.units.warrior import Warrior
from pythonwarrior.floor import Floor


class TestFloorTwoByThree(unittest.TestCase):
    def setUp(self):
        self.floor = Floor()
        self.floor.width = 2
        self.floor.height = 3

    def test_should_be_able_to_add_and_fetch_a_unit(self):
        unit = UnitBase()
        self.floor.add(unit, 0, 1, "north")
        self.assertEqual(self.floor.get(0, 1), unit)

    def test_should_not_consider_unit_on_floor_if_no_position(self):
        unit = UnitBase()
        self.floor.add(unit, 0, 1, "north")
        unit.position = None
        self.assertEqual(self.floor.units, [])

    def test_should_fetch_other_units_not_warrior(self):
        unit = UnitBase()
        warrior = Warrior()
        self.floor.add(unit, 0, 0, 'north')
        self.floor.add(warrior, 1, 0, 'north')
        self.assertNotIn(warrior, self.floor.other_units())
        self.assertIn(unit, self.floor.other_units())

    def test_should_not_consider_corner_out_of_bounds(self):
        self.assertFalse(self.floor.out_of_bounds(0, 0))
        self.assertFalse(self.floor.out_of_bounds(1, 0))
        self.assertFalse(self.floor.out_of_bounds(1, 2))
        self.assertFalse(self.floor.out_of_bounds(0, 2))

    def test_should_consider_corner_out_of_bounds_when_beyond_sides(self):
        self.assertTrue(self.floor.out_of_bounds(-1, 0))
        self.assertTrue(self.floor.out_of_bounds(0, -1))
        self.assertTrue(self.floor.out_of_bounds(0, 3))
        self.assertTrue(self.floor.out_of_bounds(2, 0))

    def test_should_return_space_at_specified_location(self):
        self.assertEqual(self.floor.space(0, 0).__class__.__name__, 'Space')

    def test_should_place_stairs_and_be_able_to_fetch_the_location(self):
        self.floor.place_stairs(1, 2)
        self.assertEqual(self.floor.stairs_location, [1, 2])


class TestFloorThreeByOne(unittest.TestCase):
    def setUp(self):
        self.floor = Floor()
        self.floor.width = 3
        self.floor.height = 1

    def test_should_print_map_with_stairs_and_unit(self):
        self.floor.add(Warrior(), 0, 0)
        self.floor.place_stairs(2, 0)
        self.assertEqual(self.floor.character, ''' ---
|@ >|
 ---
''')

    def test_should_return_unique_units(self):
        u1 = UnitBase()
        self.floor.add(u1, 0, 0)
        self.floor.add(UnitBase(), 1, 0)
        self.assertEqual(self.floor.unique_units, [u1])

if __name__ == '__main__':
    unittest.main()



from absoluteDirections import ABSOLUTE_DIRECTIONS, NORTH, SOUTH, EAST, WEST

# /**
#  * Checks if the given direction is a valid absolute direction.
#  *
#  * @param {string} direction The direction.
#  *
#  * @throws Will throw if the direction is not valid.
#  */

def verifyAbsoluteDirection(direction):
    if ( direction not in ABSOLUTE_DIRECTIONS ):
        print('Unknown Direction "{0}". Should be one of "{1}" , "{2}", "{3}", "{4}"'.format(direction, NORTH, SOUTH, EAST, WEST))

# verifyAbsoluteDirection('suth')



from verifyAbsoluteDirection import verifyAbsoluteDirection
from absoluteDirections import ABSOLUTE_DIRECTIONS
from relativeDirections import RELATIVE_DIRECTIONS

# /**
#  * Returns the relative direction for a given direction, with reference to a
#  * another direction (reference direction).
#  *
#  * @param {string} direction The direction (absolute).
#  * @param {string} referenceDirection The reference direction (absolute).
#  *
#  * @returns {string} The relative direction.
#  */

def getRelativeDirection(direction, referenceDirection):
  verifyAbsoluteDirection(direction)
  verifyAbsoluteDirection(referenceDirection)

  index = ( ABSOLUTE_DIRECTIONS.index(direction) - ABSOLUTE_DIRECTIONS.index(referenceDirection) +
      len(RELATIVE_DIRECTIONS)) % len(RELATIVE_DIRECTIONS)

  return RELATIVE_DIRECTIONS[index];


#  -----------
# |>Ca  @ S wC|
#  -----------

level.description("Time to hone your skills and apply all of the abilities that you have learned.")
level.tip("Watch your back.")
level.clue("Don't just keep shooting the bow while you are being attacked from behind.")

level.time_bonus(40)
level.ace_score(100)
level.size(11, 1)
level.stairs(0, 0)

level.warrior(5, 0, 'east')

level.unit('captive', 1, 0, 'east')
level.unit('archer', 2, 0, 'east')
level.unit('thick_sludge', 7, 0, 'west')
level.unit('wizard', 9, 0, 'west')
level.unit('captive', 10, 0, 'west')



import mock
import os
import unittest

from pythonwarrior.game import Game
from pythonwarrior.profile import Profile


class TestGame(unittest.TestCase):
    def setUp(self):
        self.game = Game()

    @mock.patch('os.mkdir')
    @mock.patch('pythonwarrior.game.UI.ask')
    def test_should_make_game_directory_if_player_says_so(self,
                                                          mock_ask,
                                                          mock_mkdir):
        mock_ask.return_value = True
        self.game.make_game_directory()
        mock_mkdir.assert_called_with('./pythonwarrior')

    @mock.patch('os.mkdir')
    @mock.patch('pythonwarrior.game.UI.ask')
    def test_should_not_make_game_and_exit_if_player_says_no(self,
                                                             mock_ask,
                                                             mock_mkdir):
        mock_ask.return_value = False
        self.assertRaises(Exception, self.game.make_game_directory)
        assert not mock_mkdir.called

    @mock.patch('pythonwarrior.game.Profile.load')
    def test_should_load_profiles_for_each_profile_path(self, mock_load):
        mock_load.side_effect = lambda x: {'foo/.profile': 1,
                                           'bar/.profile': 2}[x]
        self.game.profile_paths = mock.Mock(
            return_value=['foo/.profile', 'bar/.profile']
        )
        self.assertEqual(self.game.profiles(), [1, 2])

    @mock.patch('pythonwarrior.game.glob')
    def test_should_find_profile_paths_using_glob_search(self, mock_glob):
        self.game.profile_paths()
        mock_glob.glob.assert_called_with('./pythonwarrior/**/.profile')

    def test_should_try_to_create_profile_when_no_profile_paths(self):
        self.game.profiles = mock.Mock(return_value=[])
        self.game.new_profile = mock.Mock(return_value='profile')
        self.assertEqual(self.game.profile(), 'profile')

    @mock.patch('pythonwarrior.game.UI.choose')
    @mock.patch('pythonwarrior.game.Game.profiles')
    def test_multiple_profiles_available(self, mock_profiles, mock_choose):
        """Should prompt the user for a profile choice, but only once."""
        mock_choose.return_value = 'profile1'
        mock_profiles.return_value = ['profile1']
        self.game.profile()
        self.game.profile()
        mock_choose.assert_called_once_with('profile',
                                            ['profile1',
                                                ['new', 'New Profile']])

    @mock.patch('pythonwarrior.game.UI.choose')
    def test_ask_user_to_choose_a_tower_when_creating_new_profile(self,
                                                                  mock_choose):
        self.game.towers = mock.Mock(return_value=['tower1', 'tower2'])
        mock_choose.return_value = mock.Mock(path='/foo/bar')
        self.game.new_profile()
        mock_choose.assert_called_with('tower', ['tower1', 'tower2'])

    @mock.patch('pythonwarrior.game.UI.choose')
    @mock.patch('pythonwarrior.game.UI.request')
    @mock.patch('pythonwarrior.game.Profile')
    def test_should_pass_name_and_selected_tower_to_profile(self,
                                                            mock_profile_class,
                                                            mock_request,
                                                            mock_choose):
        mock_profile = mock.Mock()
        mock_choose.return_value = mock.Mock(path='tower_path')
        mock_request.return_value = 'name'
        mock_profile_class.return_value = mock_profile
        self.assertEqual(self.game.new_profile(), mock_profile)
        self.assertEqual(mock_profile.tower_path, 'tower_path')
        self.assertEqual(mock_profile.warrior_name, 'name')

    @mock.patch('pythonwarrior.game.Tower')
    def test_load_towers_for_each_tower_path(self, mock_tower_class):
        mock_tower_class.side_effect = lambda x: {'towers/foo': 1,
                                                  'towers/bar': 2}[x]
        self.game.tower_paths = mock.Mock(return_value=['towers/foo',
                                                        'towers/bar'])
        self.assertEqual(self.game.towers(), [1, 2])

    @mock.patch.object(os.path, 'abspath')
    @mock.patch('pythonwarrior.game.glob')
    def test_should_find_tower_paths(self, mock_glob, mock_abspath):
        mock_abspath.return_value = 'foo/foo.py'
        self.game.tower_paths()
        mock_glob.glob.assert_called_with(
            'foo/towers/*')

    def test_should_fetch_current_level_from_profile_and_cache_it(self):
        mock_level = mock.Mock(return_value='foo')
        mock_profile = mock.Mock(current_level=mock_level)
        self.game.profile = mock.Mock(return_value=mock_profile)
        self.game.current_level()
        self.game.current_level()
        mock_level.assert_called_once_with()

    def test_should_fetch_next_level_from_profile_and_cache_it(self):
        mock_level = mock.Mock(return_value='foo')
        mock_profile = mock.Mock(next_level=mock_level)
        self.game.profile = mock.Mock(return_value=mock_profile)
        self.game.next_level()
        self.game.next_level()
        mock_level.assert_called_once_with()



import mock
import unittest

from pythonwarrior.abilities.direction_of import DirectionOf


class TestDirectionOf(unittest.TestCase):
    def setUp(self):
        self.position = mock.Mock()
        self.direction_of = DirectionOf(mock.Mock(position=self.position))

    def test_should_return_relative_direction_of_given_space(self):
        self.position.relative_direction_of.return_value = 'left'
        self.assertEqual('left', self.direction_of.perform('space'))
        self.position.relative_direction_of.assert_called_once_with('space')



from verifyAbsoluteDirection import verifyAbsoluteDirection
from absoluteDirections import EAST, NORTH, SOUTH

# /**
#  * Returns the relative offset for a given location, with reference to another
#  * location (reference location) and direction (reference direction).
#  *
#  * @param {number[]} location The location.
#  * @param {number[]} referenceLocation The reference location.
#  * @param {string} referenceDirection The reference direction (absolute).
#  *
#  * @returns {number[]} The relative offset as [forward, right].
#  */

def getRelativeOffset(x1, y1, x2, y2, referenceDirection):
    verifyAbsoluteDirection(referenceDirection)

    [deltaX, deltaY] = [x1 - x2, y1 - y2]

    if ( referenceDirection == NORTH ):
        return [ -deltaY, deltaX ]
    elif ( referenceDirection == EAST ):
        return [ deltaX, deltaY ]
    elif ( referenceDirection == SOUTH ):
        return [ deltaY, -deltaX ]

    return [-deltaX, -deltaY]


import towers, warriorpyAbilities, warriorpyCli, warriorpyCore, warriorpyGeography, warriorpyUnits

__all__ = ["towers", "warriorpyAbilities", "warriorpyCli", "warriorpyCore", "warriorpyGeography", "warriorpyUnits"]



#  ----
# |ssC>|
# |@sss|
# |ssC |
#  ----

level.description("Never before have you seen a room so full of sludge. "
                  "Start the fireworks!")
level.tip("Be careful not to let the ticking captive get caught in "
          "the flames. Use warrior.distance_of to avoid the captives.")
level.clue("Be sure to bind the surrounding enemies before fighting. "
           "Check your health before detonating explosives.")
level.time_bonus(70)
level.size(4, 3)
level.stairs(3, 0)

def add_war_abilities(warrior):
    warrior.add_abilities('distance_of')

level.warrior(0, 1, 'east', func=add_war_abilities)

def add_captive_abilities(unit):
    unit.add_abilities('explode_')
    unit.abilities_attr['explode_'].time = 20

level.unit('captive', 2, 0, 'south', func=add_captive_abilities)
level.unit('captive', 2, 2, 'north')

level.unit('sludge', 0, 0, 'south')
level.unit('sludge', 1, 0, 'south')
level.unit('sludge', 1, 1, 'east')
level.unit('sludge', 2, 1, 'east')
level.unit('sludge', 3, 1, 'east')
level.unit('sludge', 0, 2, 'north')
level.unit('sludge', 1, 2, 'north')



import mock
import unittest

from pythonwarrior.abilities.distance_of import DistanceOf


class TestDistanceOf(unittest.TestCase):
    def setUp(self):
        self.position = mock.Mock()
        self.distance = DistanceOf(mock.Mock(position=self.position))

    def test_should_return_distance_from_stairs(self):
        self.position.distance_of.return_value = 5
        self.assertEqual(5, self.distance.perform('space'))
        self.position.distance_of.assert_called_once_with('space')



import re

import warriorpyAbilities.src.attack.Attack as Attack
import warriorpyAbilities.src.bind.Bind as Bind
import warriorpyAbilities.src.detonate.Detonate as Detonate
import warriorpyAbilities.src.direction_of.DirectionOf as DirectionOf
import warriorpyAbilities.src.direction_of_stairs.DirectionOfStairs as DirectionOfStairs
import warriorpyAbilities.src.distance_of.DistanceOf as DistanceOf
import warriorpyAbilities.src.explode.Explode as Explode
import warriorpyAbilities.src.feel.Feel as Feel
import warriorpyAbilities.src.health.Health as Health
import warriorpyAbilities.src.listen.Listen as Listen
import warriorpyAbilities.src.look.Look as Look
import warriorpyAbilities.src.pivot.Pivot as Pivot
import warriorpyAbilities.src.rescue.Rescue as Rescue
import warriorpyAbilities.src.rest.Rest as Rest
import warriorpyAbilities.src.shoot.Shoot as Shoot
import warriorpyAbilities.src.walk.Walk as Walk
import warriorpyCore.src.turn.Turn as Turn
import warriorpyCli.src.ui.ui.UI as UI


class UnitBase(object):
    def __init__(self):
        self.position = None
        self._health = None
        self.abilities_attr = None
        self.bound = False
        self.max_health = 0

    @property
    def abilities(self):
        if not self.abilities_attr:
            self.abilities_attr = {}
        return self.abilities_attr

    def __repr__(self):
        return self.name()

    @property
    def attack_power(self):
        return 0

    def is_alive(self):
        return self.position is not None

    def earn_points(self, points):
        pass

    @property
    def health(self):
        if self._health is None:
            self._health = self.max_health
        return self._health

    @health.setter
    def health(self, value):
        self._health = value

    def take_damage(self, amount):
        if self.is_bound():
            self.unbind()
        if self.health:
            self._health -= amount
            self.say("takes %(amount)d damage, "
                     "%(health)d health power left" %
                     {'amount': amount, 'health': self.health})
            if self.health <= 0:
                self.position = None
                self.say("dies")

    def name(self):
        return self.__class__.__name__

    def prepare_turn(self):
        self.current_turn = self.next_turn()
        self.play_turn(self.current_turn)

    def next_turn(self):
        return Turn(self.abilities)

    def play_turn(self, turn):
        return None

    def is_bound(self):
        return self.bound

    def add_abilities(self, *new_abilities):
        for ability in new_abilities:
            class_name = re.sub("_", "", ability.title())
            self.abilities[ability] = eval("%s(self)" % class_name)

    def say(self, msg):
        UI.puts_with_delay("%(name)s %(msg)s" % {'name': self.name(),
                                                 'msg': msg})

    @property
    def character(self):
        return "?"

    def bind(self):
        self.bound = True

    def unbind(self):
        self.say("released from bonds")
        self.bound = False

    def perform_turn(self):
        if self.position:
            for ability in self.abilities.values():
                ability.pass_turn()
            if self.current_turn.action and not self.is_bound():
                name = self.current_turn.action[0]
                args = self.current_turn.action[1:]
                self.abilities[name].perform(*args)



from .base import AbilityBase


class Look(AbilityBase):
    def description(self):
        return("Returns an array of up to three Spaces in the given direction",
               "(forward by default)")

    def perform(self, direction='forward'):
        self.verify_direction(direction)
        return map(lambda amount: self.space(direction, amount), [1, 2, 3])



import unittest

from pythonwarrior.units.warrior import Warrior
from pythonwarrior.abilities.health import Health


class TestHealth(unittest.TestCase):
    def setUp(self):
        self.warrior = Warrior()
        self.health = Health(self.warrior)

    def test_should_return_the_amount_of_health(self):
        self.warrior.health = 10
        self.assertEqual(10, self.health.perform())



from .base import UnitBase


class Golem(UnitBase):
    def __init__(self):
        super(Golem, self).__init__()
        self.turn = None
        self.max_health = None

    def play_turn(self, turn):
        if self.turn:
            self.turn(turn)

    @property
    def character(self):
        return "G"

    @property
    def attack_power(self):
        return 3



import mock
import unittest

from pythonwarrior.units.golem import Golem


class TestGolem(unittest.TestCase):
    def setUp(self):
        self.golem = Golem()

    def test_should_execute_turn_proc_when_playing_turn(self):
        proc = mock.Mock()
        self.golem.turn = proc
        self.golem.play_turn('turn')
        proc.assert_called_once_with('turn')

    def test_should_set_max_health_and_update_current_health(self):
        self.golem.max_health = 10
        self.assertEqual(self.golem.max_health, 10)
        self.assertEqual(self.golem.health, 10)

    def test_should_have_attack_power_of_3(self):
        self.assertEqual(self.golem.attack_power, 3)

    def test_should_appear_as_G_on_map(self):
        self.assertEqual(self.golem.character, 'G')



import functools


class Turn(object):
    def __init__(self, abilities):
        self.action = None
        self.senses = {}

        for name, sense in abilities.iteritems():
            if name.endswith("_"):
                self.add_action(name)
            else:
                self.add_sense(name, sense)

    def add_action(self, action):
        self.__dict__[action] = functools.partial(self._generic_action, action)

    def add_sense(self, name, sense):
        self.senses[name] = sense
        self.__dict__[name] = functools.partial(self._generic_sense, name)

    def _generic_action(self, action_name, *args):
        if self.action:
            raise Exception("Only one action can be performed per turn")
        self.action = [action_name]
        for arg in args:
            self.action.append(arg)

    def _generic_sense(self, sense_name, *args):
        return self.senses[sense_name].perform(*args)



import warriorpyCore.config.Config as Config
import time


class UI(object):
    @staticmethod
    def puts(msg):
        if Config.out_stream:
            return Config.out_stream.write(msg + "\n")

    @staticmethod
    def puts_with_delay(msg):
        result = UI.puts(msg)
        if Config.delay:
            time.sleep(Config.delay)
        return result

    @staticmethod
    def write(msg):
        if Config.out_stream:
            return Config.out_stream.write(msg)

    @staticmethod
    def gets():
        if Config.in_stream:
            return Config.in_stream.readline()
        else:
            return ''

    @staticmethod
    def request(msg):
        UI.write(msg)
        return UI.gets().rstrip()

    @staticmethod
    def ask(msg):
        return UI.request("%s [yn] " % msg) == 'y'

    @staticmethod
    def choose(item, options):
        if len(options) == 1:
            response = options[0]
        else:
            for idx, option in enumerate(options):
                if type(option) == list:
                    UI.puts("[%d] %s" % (idx+1, option[-1]))
                else:
                    UI.puts("[%d] %s" % (idx+1, option))
            choice = UI.request("Choose %s by typing the number: " % item)
            response = options[int(choice)-1]
        if type(response) == list:
            return response[0]
        else:
            return response



import unittest

from pythonwarrior.units.sludge import Sludge


class TestSludge(unittest.TestCase):
    def setUp(self):
        self.sludge = Sludge()

    def test_should_have_attack_action(self):
        self.assertIn('attack_', self.sludge.abilities.keys())

    def test_should_have_feel_sense(self):
        self.assertIn('feel', self.sludge.abilities.keys())

    def test_should_have_attack_of_3(self):
        self.assertEqual(self.sludge.attack_power, 3)

    def test_should_have_12_max_health(self):
        self.assertEqual(self.sludge.max_health, 12)

    def test_should_appear_as_s_on_map(self):
        self.assertEqual(self.sludge.character, "s")



from .position import Position
from .space import Space


class Floor(object):
    def __init__(self):
        self.width = 0
        self.height = 0
        self._units = []
        self.stairs_location = [-1, -1]

    def add(self, unit, x, y, direction=None):
        self._units.append(unit)
        unit.position = Position(self, x, y, direction)

    def place_stairs(self, x, y):
        self.stairs_location = [x, y]

    def stairs_space(self):
        return self.space(*self.stairs_location)

    @property
    def units(self):
        return filter(lambda unit: unit.position is not None, self._units)

    def other_units(self):
        return filter(lambda unit: unit.__class__.__name__ != 'Warrior',
                      self.units)

    def get(self, x, y):
        for unit in self.units:
            if unit.position.at(x, y):
                return unit

    def space(self, x, y):
        return Space(self, x, y)

    def out_of_bounds(self, x, y):
        return x < 0 or y < 0 or x > self.width-1 or y > self.height-1

    @property
    def character(self):
        rows = []
        rows.append(" " + ("-" * self.width))
        for y in range(self.height):
            row = "|"
            for x in range(self.width):
                row += self.space(x, y).character
            row += "|"
            rows.append(row)
        rows.append(" " + ("-" * self.width))
        return "\n".join(rows) + "\n"

    @property
    def unique_units(self):
        uniq_units = []
        for unit in self.units:
            uniq_classes = map(lambda unit: unit.__class__.__name__,
                               uniq_units)
            uniq_classes = list(set(uniq_classes))
            if not unit.__class__.__name__ in uniq_classes:
                uniq_units.append(unit)

        return uniq_units



import os
import sys


root_dir = os.path.normpath(
    os.path.join(
        os.path.abspath(sys.argv[0]),
        os.pardir,
        os.pardir,
    )
)
sys.path.insert(0, root_dir)

import warriorpy.runner as runner


runner = runner.Runner(sys.argv, sys.stdin, sys.stdout)
runner.run()


import mock
import unittest

from pythonwarrior.abilities.bind import Bind
from pythonwarrior.units.base import UnitBase


class TestBind(unittest.TestCase):
    def setUp(self):
        self.bind = Bind(mock.Mock())

    def test_should_bind_recipient(self):
        receiver = UnitBase()
        self.bind.unit = mock.Mock(return_value=receiver)
        self.bind.perform()
        self.assertTrue(receiver.is_bound())

    def test_should_do_nothing_if_no_recipient(self):
        self.bind.unit = mock.Mock(return_value=None)
        self.bind.perform()



import unittest
import mock
from pythonwarrior.units.base import UnitBase


class TestUnitBase(unittest.TestCase):
    def setUp(self):
        self.unit = UnitBase()

    def test_should_have_attack_of_zero(self):
        self.assertEqual(self.unit.attack_power, 0)

    def test_should_be_dead_if_no_position(self):
        self.assertEqual(self.unit.position, None)
        self.assertFalse(self.unit.is_alive())

    def test_should_be_alive_if_has_position(self):
        self.unit.position = mock.Mock()
        self.assertTrue(self.unit.is_alive())

    def test_should_default_max_health_to_0(self):
        self.assertEqual(self.unit.max_health, 0)

    def test_should_default_health_to_max_health(self):
        self.unit.max_health = 10
        self.assertEqual(self.unit.health, 10)

    def test_should_subtract_health_when_taking_damage(self):
        self.unit.max_health = 10
        self.unit.take_damage(3)
        self.assertEqual(self.unit.health, 7)

    def test_should_set_position_to_none_when_out_of_health(self):
        self.max_health = 10
        self.unit.take_damage(10)
        self.assertEqual(self.unit.position, None)

    def test_should_return_name_as_string_representation(self):
        self.assertEqual(self.unit.name(), 'UnitBase')
        self.assertEqual(str(self.unit), 'UnitBase')

    @mock.patch('pythonwarrior.units.base.UnitBase.next_turn')
    @mock.patch('pythonwarrior.units.base.UnitBase.play_turn')
    def test_should_prepare_turn_by_calling_play_turn_with_next_turn_object(
            self, mock_play_turn, mock_next_turn):
        mock_next_turn.return_value = "next_turn"
        self.unit.prepare_turn()
        mock_play_turn.assert_called_with('next_turn')

    @mock.patch('pythonwarrior.units.base.UnitBase.next_turn')
    @mock.patch('pythonwarrior.units.base.Walk')
    def test_should_perform_action_when_calling_perform_on_turn(
            self, mock_walk, mock_next_turn):
        self.unit.position = mock.Mock()

        walk_object = mock.Mock(name='asdfs')
        mock_walk.return_value = walk_object

        self.unit.add_abilities('walk_')

        turn = mock.Mock(action=['walk_', 'backward'])
        mock_next_turn.return_value = turn

        self.unit.prepare_turn()
        self.unit.perform_turn()
        walk_object.perform.assert_called_once_with('backward')

    @mock.patch('pythonwarrior.units.base.UnitBase.next_turn')
    @mock.patch('pythonwarrior.units.base.Walk')
    def test_should_not_form_action_when_dead(self, mock_walk, mock_next_turn):
        self.unit.position = None

        walk_object = mock.Mock()
        mock_walk.return_value = walk_object
        walk_object.perform.side_effect = Exception('shouldnt be called')

        self.unit.add_abilities('walk_')
        turn = mock.Mock(action=['walk_', 'backward'])
        mock_next_turn.return_value = turn

        self.unit.prepare_turn()
        self.unit.perform_turn()

    def test_should_not_raise_exception_when_perform_turn_with_no_action(self):
        self.unit.prepare_turn()
        self.unit.perform_turn()

    @mock.patch('pythonwarrior.units.base.UnitBase.abilities')
    @mock.patch('pythonwarrior.units.base.Turn')
    def test_should_pass_abilities_to_new_turn_when_calling_next_turn(
            self, mock_turn, mock_abilities):
        mock_turn.return_value = 'turn'
        self.assertEqual(self.unit.next_turn(), 'turn')
        mock_turn.assert_called_once_with(mock_abilities)

    @mock.patch('pythonwarrior.units.base.Walk')
    def test_should_add_abilities_to_abilities_dict(self, mock_walk):
        mock_walk.return_value = 'walk'
        self.unit.add_abilities("walk_")
        self.assertEqual(self.unit.abilities, {'walk_': 'walk'})

    def test_should_appear_as_question_mark_on_map(self):
        self.assertEqual(self.unit.character, "?")

    def test_should_be_released_from_bonds_when_taking_damage(self):
        self.unit.max_health = 10
        self.unit.bind()
        self.assertTrue(self.unit.is_bound())
        self.unit.take_damage(2)
        self.assertFalse(self.unit.is_bound())

    def test_should_be_released_from_bonds_when_calling_release(self):
        self.unit.bind()
        self.unit.unbind()
        self.assertFalse(self.unit.is_bound())

    @mock.patch('pythonwarrior.units.base.UnitBase.next_turn')
    def test_should_not_perform_action_when_bound(self, mock_next_turn):
        self.unit.position = mock.Mock()
        self.unit.bind()
        self.unit.add_abilities("walk_")
        turn = mock.Mock()
        turn.action = ["walk_", "backward"]
        mock_next_turn.return_value = turn
        self.unit.prepare_turn()
        self.unit.perform_turn()


if __name__ == '__main__':
    unittest.main()



from .base import AbilityBase


class Explode(AbilityBase):
    def description(self):
        return("Kills you and all surrounding units."
               " You probably don't want to do this intentionally.")

    def perform(self):
        if self._unit.position:
            self._unit.say("explodes, collapsing the ceiling and "
                           "damaging every unit.")
            for unit in self._unit.position.floor.units:
                unit.take_damage(100)

    def pass_turn(self):
        if self.time and self._unit.position:
            self._unit.say("is ticking")
            self.time -= 1
            if self.time == 0:
                self.perform()



from .base import UnitBase


class Captive(UnitBase):
    def __init__(self):
        super(Captive, self).__init__()
        self.bind()
        self.max_health = 1

    @property
    def character(self):
        return "C"



#  -------
# |@ CaaSC|
#  -------

level.description("You hear cries for help. Captives must need rescuing.")
level.tip("Use warrior.feel().is_captive() to see if there is a captive and warrior.rescue_() to rescue him. Don't attack captives.")
level.clue("Don't forget to constantly check if you're taking damage. Rest until your health is full if you aren't taking damage.")

level.time_bonus(45)
level.ace_score(123)
level.size(7, 1)
level.stairs(6, 0)

def add_abilities(warrior):
    warrior.add_abilities('rescue_')

level.warrior(0, 0, 'east', func=add_abilities)

level.unit('captive', 2, 0, 'west')
level.unit('archer', 3, 0, 'west')
level.unit('archer', 4, 0, 'west')
level.unit('thick_sludge', 5, 0, 'west')
level.unit('captive', 6, 0, 'west')



import base64
import mock
import pickle
import unittest
from pythonwarrior.profile import Profile


class TestProfile(unittest.TestCase):
    def setUp(self):
        self.profile = Profile()

    def test_should_have_warrior_name(self):
        self.profile.warrior_name = "Joe"
        self.assertEqual(self.profile.warrior_name, "Joe")

    def test_should_start_level_number_at_0(self):
        self.assertEqual(self.profile.level_number, 0)

    def test_should_start_score_at_0_and_allow_it_to_increment(self):
        self.assertEqual(self.profile.score, 0)
        self.profile.score += 5
        self.assertEqual(self.profile.score, 5)

    def test_should_have_no_abilities_and_allow_adding(self):
        self.assertEqual(self.profile.abilities, [])
        self.profile.abilities += ['foo', 'bar']
        self.assertEqual(self.profile.abilities, ['foo', 'bar'])

    def test_should_encode_with_pickle_and_base64(self):
        self.assertEqual(self.profile.encode(),
                         base64.b64encode(pickle.dumps(self.profile)))

    def test_should_decode_with_pickle_and_base64(self):
        self.assertEqual(Profile.decode(self.profile.encode()).warrior_name,
                         self.profile.warrior_name)

    @mock.patch('__builtin__.open')
    def test_load_should_read_file_decode_and_set_player_path(self, mock_open):
        profile = mock.Mock()
        mock_open.read.return_value = "encoded_profile"
        with mock.patch('pythonwarrior.profile.Profile.decode',
                        return_value=profile):
            self.assertEqual(Profile.load('path/to/.profile'), profile)
            mock_open.assert_called_once_with("path/to/.profile")

    def test_should_add_abilities_and_remove_duplicates(self):
        self.profile.add_abilities('foo', 'bar', 'blah', 'bar')
        self.assertItemsEqual(self.profile.abilities, ['foo', 'bar', 'blah'])

    def test_should_fetch_new_level_with_current_number(self):
        self.profile.level_number = 1
        self.assertEqual(self.profile.current_level().number, 1)

    def test_should_fetch_next_level(self):
        self.profile.level_number = 1
        self.assertEqual(self.profile.next_level().number, 2)

    def test_should_enable_epic_mode_and_reset_scores_if_none(self):
        self.profile.enable_epic_mode()
        self.assertTrue(self.profile.epic)
        self.assertEqual(0, self.profile.epic_score)
        self.assertEqual(0, self.profile.current_epic_score)

    def test_should_override_epic_score_with_current_one_if_it_is_higher(self):
        self.profile.enable_epic_mode()
        self.assertEqual(0, self.profile.epic_score)
        self.assertEqual(None, self.profile.average_grade)
        self.profile.current_epic_score = 123
        self.profile.current_epic_grades = {1: 0.7, 2: 0.9}
        self.profile.update_epic_score()
        self.assertEqual(123, self.profile.epic_score)
        self.assertEqual(0.8, self.profile.average_grade)

    def test_should_not_override_epic_score_with_current_one_if_lower(self):
        self.profile.enable_epic_mode()
        self.profile.epic_score = 124
        self.profile.average_grade = 0.9
        self.profile.current_epic_score = 123
        self.profile.current_epic_grades = {1: 0.7, 2: 0.9}
        self.profile.update_epic_score()
        self.assertEqual(124, self.profile.epic_score)
        self.assertEqual(0.9, self.profile.average_grade)

    def test_should_not_calculate_average_grade_if_no_grades_are_present(self):
        self.profile.enable_epic_mode()
        self.profile.current_epic_grades = {}
        self.assertIsNone(self.profile.calculate_average_grade())


class TestProfileWithTowerPath(unittest.TestCase):
    def setUp(self):
        self.profile = Profile()
        self.profile.warrior_name = "John Smith"
        self.profile.tower_path = "path/to/tower"

    def test_save_should_write_file_with_encoded_profile(self):
        with mock.patch('__builtin__.open') as mock_open:
            with mock.patch.object(self.profile, 'encode',
                                   return_value='encoded_profile'):
                f = mock.Mock()
                mock_open.return_value = f
                self.profile.save()
                f.write.assert_called_once_with('encoded_profile')
                mock_open.assert_called_once_with(self.profile._player_path +
                                                  '/.profile', 'w')

    def test_should_have_a_nice_string_representation(self):
        self.profile.warrior_name = "Joe"
        self.assertEqual(str(self.profile), "Joe - tower - level 0 - score 0")

    def test_should_guess_at_the_player_path(self):
        self.assertEqual(self.profile.player_path,
                         './pythonwarrior/john-smith-tower')

    def test_should_use_specified_player_path(self):
        self.profile._player_path = "path/to/player"
        self.assertEqual(self.profile.player_path, "path/to/player")

    def test_should_load_tower_from_path(self):
        with mock.patch('pythonwarrior.profile.Tower') as mock_tower:
            tower = mock.Mock()
            mock_tower.return_value = tower
            self.assertEqual(self.profile.tower(), tower)



import warriorpyCore.src.position.Position as Position
import warriorpyGeography.src.getAbsoluteOffset.getAbsoluteOffset as getAbsoluteOffset
import warriorpyGeography.src.verifyAbsoluteDirection.verifyAbsoluteDirection as verifyAbsoluteDirection

class AbilityBase(object):
    def __init__(self, unit):
        self._unit = unit

    def offset(self, direction, forward=1, right=0):
        getAbsoluteOffset(forward, right, direction)

    def verify_direction(self, direction):
        verifyAbsoluteDirection(direction)

    def space(self, direction, forward=1, right=0):
        offset = self.offset(direction, forward, right)
        return self._unit.position.relative_space(*offset)

    def unit(self, direction, forward=1, right=0):
        return self.space(direction, forward, right).unit

    def damage(self, receiver, amount):
        receiver.take_damage(amount)
        if not receiver.is_alive():
            self._unit.earn_points(receiver.max_health)

    def description(self):
        return None

    def pass_turn(self):
        return None

    



import unittest
from pythonwarrior.abilities.explode import Explode
from pythonwarrior.floor import Floor
from pythonwarrior.units.captive import Captive
from pythonwarrior.units.base import UnitBase


class TestExplode(unittest.TestCase):
    def setUp(self):
        self.floor = Floor()
        self.floor.width = 2
        self.floor.height = 3
        self.captive = Captive()
        self.floor.add(self.captive, 0, 0)
        self.explode = Explode(self.captive)

    def test_should_subtract_100_health_from_each_unit_on_the_floor(self):
        unit = UnitBase()
        unit._health = 20
        self.floor.add(unit, 0, 1)
        self.captive._health = 10
        self.explode.perform()
        self.assertEqual(self.captive.health, -90)
        self.assertEqual(unit.health, -80)

    def test_should_explode_when_bomb_time_reaches_zero(self):
        self.captive._health = 10
        self.explode.time = 3
        self.explode.pass_turn()
        self.explode.pass_turn()
        self.assertEqual(self.captive.health, 10)
        self.explode.pass_turn()
        self.assertEqual(self.captive.health, -90)



from .base import AbilityBase


class Detonate(AbilityBase):
    def description(self):
        return ("Detonate a bomb in a given direction (forward by default) "
                "which damages that space and surrounding 4 spaces "
                "(including yourself).")

    def perform(self, direction='forward'):
        self.verify_direction(direction)
        if self._unit.position:
            self._unit.say("detonates a bomb %s launching "
                           "a deadly explosion" % direction)
            self.bomb(direction, 1, 0, 8)
            for x, y in [(1, 1), (1, -1), (2, 0), (0, 0)]:
                self.bomb(direction, x, y, 4)

    def bomb(self, direction, x, y, damage_amount):
        if self._unit.position:
            receiver = self.space(direction, x, y).unit
            if receiver:
                if receiver.abilities.get('explode_'):
                    receiver.say("caught in bomb's flames which "
                                 "detonates ticking explosive")
                    receiver.abilities['explode_'].perform()
                else:
                    self.damage(receiver, damage_amount)



from verifyAbsoluteDirection import verifyAbsoluteDirection
from absoluteDirections import EAST, NORTH, SOUTH

# /**
#  * Returns the absolute offset for a given relative offset with reference
#  * to a given direction (reference direction).
#  *
#  * @param {number[]} offset The relative offset as [forward, right].
#  * @param {string} referenceDirection The reference direction (absolute).
#  *
#  * @returns {number[]} The absolute offset as [deltaX, deltaY].
#  */

def getAbsoluteOffset(forward, right, referenceDirection):
  verifyAbsoluteDirection(referenceDirection)

  if (referenceDirection == NORTH):
    return [right, -forward]
  elif (referenceDirection == EAST):
    return [forward, right] 
  elif (referenceDirection == SOUTH):
    return [-right, forward]

  return [-forward, -right];


from .floor import Floor

# These are used dynamically via unit_to_constant
import warriorpyUnits.src.archer.Archer as Archer
import warriorpyUnits.src.captive.Captive as Captive
import warriorpyUnits.src.golem.Golem as Golem
import warriorpyUnits.src.sludge.Sludge as Sludge
import warriorpyUnits.src.thick_sludge.ThickSludge as ThickSludge
import warriorpyUnits.src.wizard.Wizard as Wizard

import warriorpyUnits.src.base.UnitBase as UnitBase
import warriorpyUnits.src.warrior.Warrior as Warrior


class LevelLoader(object):
    def __init__(self, level):
        self.floor = Floor()
        self.level = level
        self.level.floor = self.floor

    def description(self, desc):
        self.level.description = desc

    def tip(self, level_tip):
        self.level.tip = level_tip

    def clue(self, level_clue):
        self.level.clue = level_clue

    def time_bonus(self, bonus):
        self.level.time_bonus = bonus

    def ace_score(self, score):
        self.level.ace_score = score

    def size(self, width, height):
        self.floor.width = width
        self.floor.height = height

    def stairs(self, x, y):
        self.floor.place_stairs(x, y)

    def unit(self, a_unit, x, y, facing="north", func=None):
        if not isinstance(a_unit, UnitBase):
            a_unit = self._unit_to_constant(a_unit)()

        self.floor.add(a_unit, x, y, facing)
        if func:
            func(a_unit)
        return a_unit

    def warrior(self, *args, **kwargs):
        a_func = kwargs.get('func', None)
        warrior = Warrior(self.level)
        return self.level.setup_warrior(self.unit(warrior, *args, func=a_func))

    @staticmethod
    def _unit_to_constant(name):
        camel = "".join(map(lambda x: x.capitalize(), str(name).split('_')))
        return eval(camel)



import mock
import unittest
from pythonwarrior.turn import Turn
from pythonwarrior.abilities.feel import Feel


class TestTurnWithActions(unittest.TestCase):
    def setUp(self):
        self.turn = Turn({'walk_': None, 'attack_': None})

    def test_should_have_no_action_performed_at_first(self):
        self.assertIsNone(self.turn.action)

    def test_should_be_able_to_perform_action_and_recall_it(self):
        self.turn.walk_()
        self.assertEqual(self.turn.action, ['walk_'])

    def test_should_include_arguments_passed_to_action(self):
        self.turn.walk_('forward')
        self.assertEqual(self.turn.action, ['walk_', 'forward'])

    def test_should_not_be_able_to_call_multiple_actions_per_turn(self):
        self.turn.walk_('forward')
        self.assertRaises(Exception, self.turn.attack_)


class TestTurnWithSenses(unittest.TestCase):
    def setUp(self):
        with mock.patch('pythonwarrior.abilities.feel.Feel.space',
                        mock.Mock()):
            self.feel = Feel(mock.Mock())
            self.turn = Turn({'feel': self.feel})

    def test_call_sense_with_any_argument_and_return_expected_results(self):
        self.assertEqual(self.turn.feel(), self.feel.perform())
        self.assertEqual(self.turn.feel('backward'),
                         self.feel.perform('backward'))



#  -------
# |@ Ss C>|
#  -------

level.description("You discover a satchel of bombs which will help "
                  "when facing a mob of enemies.")
level.tip("Detonate a bomb when you see a couple enemies ahead of "
          "you (warrior.look()). Watch out for your health too.")
level.clue("Calling warrior.look() will return an array of Spaces. If the "
           "first two contain enemies, detonate a bomb with "
           "warrior.detonate_().")
level.time_bonus(30)
level.size(7, 1)
level.stairs(6, 0)

def add_war_abilities(warrior):
    warrior.add_abilities('look', 'detonate_')

level.warrior(0, 0, 'east', func=add_war_abilities)

def add_captive_abilities(unit):
    unit.add_abilities('explode_')
    unit.abilities_attr['explode_'].time = 9

level.unit('captive', 5, 0, 'west', func=add_captive_abilities)
level.unit('thick_sludge', 2, 0, 'west')
level.unit('sludge', 3, 0, 'west')



from verifyRelativeDirection import verifyRelativeDirection
from relativeDirections import BACKWARD, FORWARD, RIGHT

# /**
#  * Rotates the given relative offset in the given direction.
#  *
#  * @param {number[]} offset The relative offset as [forward, right].
#  * @param {string} direction The direction (relative direction).
#  *
#  * @returns {number[]} The rotated offset as [forward, right].
#  */

def rotateRelativeOffset(forward, right, direction):
  verifyRelativeDirection(direction);

  if (direction == FORWARD):
    return [forward, right]
  elif (direction == RIGHT):
    return [-right, forward]
  elif (direction == BACKWARD):
    return [-forward, -right]

  return [right, -forward];


from .base import AbilityBase


class Health(AbilityBase):
    def description(self):
        return "Returns an integer representing your health."

    def perform(self):
        return self._unit.health



import glob
import os
import sys

from .level_loader import LevelLoader
import warriorpycli.src.player_generator.PlayerGenerator as PlayerGenerator
import warriorpycli.src.ui.ui.UI as UI


class Level(object):
    @staticmethod
    def grade_letter(percent):
        if percent >= 1.0:
            return 'S'
        elif percent >= 0.9:
            return 'A'
        elif percent >= 0.8:
            return 'B'
        elif percent >= 0.7:
            return 'C'
        elif percent >= 0.6:
            return 'D'
        else:
            return 'F'

    def __init__(self, profile, number):
        self.profile = profile
        self.number = number
        self.time_bonus = 0
        self.ace_score = None
        self.clue = None
        self.warrior = None
        self.description = None
        self.tip = None
        self.floor = None

    def player_path(self):
        return self.profile.player_path

    def load_path(self):
        return os.path.join(
            os.path.normpath(os.path.abspath(__file__) + '../../towers'),
            os.path.basename(self.profile.tower_path) +
            '/level_' + str(self.number).rjust(3, '0') +
            '.py'
        )

    def load_level(self):
        level = LevelLoader(self)
        f = open(self.load_path())
        exec(f.read())

    def generate_player_files(self):
        self.load_level()
        PlayerGenerator(self, Level(self.profile, self.number-1)).generate()

    def play(self, turns=1000, fxn=None):
        self.load_level()
        for turn in range(turns):
            if self.is_passed() or self.is_failed():
                return
            UI.puts('- turn %d -' % (turn+1))
            UI.write(self.floor.character)
            for unit in self.floor.units:
                unit.prepare_turn()
            for unit in self.floor.units:
                unit.perform_turn()
            if fxn:
                fxn()
            if self.time_bonus > 0:
                self.time_bonus -= 1

    def tally_points(self):
        score = 0

        UI.puts('Level Score: %d' % self.warrior.score)
        score += self.warrior.score

        UI.puts('Time Bonus: %d' % self.time_bonus)
        score += self.time_bonus

        if not self.floor.other_units():
            UI.puts('Clear Bonus: %d' % self.clear_bonus())
            score += self.clear_bonus()

        if self.profile.epic:
            if self.grade_for(score):
                UI.puts('Level Grade: %s' % self.grade_for(score))
            UI.puts('Total Score: %s' %
                    self.score_calculation(self.profile.current_epic_score,
                                           score))
            if self.ace_score:
                grade = (score / float(self.ace_score))
                self.profile.current_epic_grades[self.number] = grade
            self.profile.current_epic_score += score
        else:
            UI.puts('Total Score: %s' %
                    self.score_calculation(self.profile.score, score))
            self.profile.score += score
            self.profile.abilities = self.warrior.abilities.keys()

    def grade_for(self, score):
        if self.ace_score:
            return Level.grade_letter(score/float(self.ace_score))

    def clear_bonus(self):
        return round((self.warrior.score + self.time_bonus) * 0.2)

    def score_calculation(self, current_score, addition):
        if current_score == 0:
            return str(addition)
        else:
            return '%d + %d = %d' % (current_score, addition,
                                     (current_score + addition))

    def is_passed(self):
        return self.floor.stairs_space().is_warrior()

    def is_failed(self):
        return self.warrior not in self.floor.units

    def exists(self):
        return os.path.exists(self.load_path())

    def setup_warrior(self, warrior):
        self.warrior = warrior
        self.warrior.add_abilities(*self.profile.abilities)
        self.warrior.name_attr = self.profile.warrior_name



# /**
#  * Translates the given location by a given offset.
#  *
#  * @param {number[]} location The location as [x, y].
#  * @param {number[]} offset The offset as [deltaX, deltaY].
#  *
#  * @returns {number[]} The translated location.
#  */

def translateLocation(x, y, deltaX, deltaY):
  return [x + deltaX, y + deltaY]


import mock
import unittest

from pythonwarrior.abilities.look import Look


class TestLook(unittest.TestCase):
    def setUp(self):
        self.unit = mock.Mock()
        self.look = Look(self.unit)

    def test_should_get_3_objects_at_position_from_offset(self):
        def fake_rel_space(x, y):
            mapping = {(1, 0): 1,
                       (2, 0): 2,
                       (3, 0): 3}
            return mapping[(x, y)]
        mock_rel_space = mock.Mock(side_effect=fake_rel_space)
        self.unit.position.relative_space = mock_rel_space

        self.assertEqual([1, 2, 3], self.look.perform('forward'))
        self.assertEqual(3, mock_rel_space.call_count)



#  ------
# |Cs   >|
# |@  sC |
#  ------

level.description("What's that ticking? Some captives have a timed bomb "
                  "at their feet!")
level.tip("Hurry and rescue captives first that have space.is_ticking(), "
          "they'll soon go!")
level.clue("Avoid fighting enemies at first. Use warrior.listen() and "
           "space.is_ticking() and quickly rescue those captives.")
level.time_bonus(50)
level.ace_score(108)
level.size(6, 2)
level.stairs(5, 0)

level.warrior(0, 1, 'east')

level.unit('sludge', 1, 0, 'west')
level.unit('sludge', 3, 1, 'west')
level.unit('captive', 0, 0, 'west')

def add_abilities(unit):
    unit.add_abilities('explode_')
    unit.abilities_attr['explode_'].time = 7

level.unit('captive', 4, 1, 'west', func=add_abilities)



class Space(object):
    def __init__(self, floor, x, y):
        self.floor = floor
        self.x = x
        self.y = y

    def __repr__(self):
        if self.unit:
            return str(self.unit)
        elif self.is_wall():
            return 'wall'
        else:
            return 'nothing'

    def is_wall(self):
        return self.floor.out_of_bounds(self.x, self.y)

    def is_warrior(self):
        return self.unit.__class__.__name__ == "Warrior"

    def is_golem(self):
        return self.unit.__class__.__name__ == "Golem"

    def is_player(self):
        return self.is_warrior() or self.is_golem()

    def is_enemy(self):
        return self.unit and not self.is_player() and not self.is_captive()

    def is_captive(self):
        return self.unit and self.unit.is_bound()

    def is_empty(self):
        return self.unit is None and not self.is_wall()

    def is_stairs(self):
        return self.floor.stairs_location == self.location()

    def is_ticking(self):
        return self.unit and 'explode_' in self.unit.abilities

    @property
    def unit(self):
        return self.floor.get(self.x, self.y)

    def location(self):
        return [self.x, self.y]

    @property
    def character(self):
        if self.unit:
            return self.unit.character
        elif self.is_stairs():
            return ">"
        else:
            return " "



from .base import UnitBase


class Archer(UnitBase):
    def __init__(self):
        super(Archer, self).__init__()
        self.max_health = 7
        self.add_abilities('shoot_', 'look')

    @property
    def shoot_power(self):
        return 3

    @property
    def character(self):
        return 'a'

    def play_turn(self, turn):
        for direction in ['forward', 'left', 'right']:
            for space in turn.look(direction):
                if space.is_player():
                    turn.shoot_(direction)
                    return
                elif not space.is_empty():
                    break



#  ------
# |      |
# |@     |
# |      |
# |  >   |
#  ------

level.description("Silence. The room feels large, but empty. "
                  "Luckily you have a map of this tower to help "
                  "find the stairs.")
level.tip("Use warrior.direction_of_stairs to determine which "
          "direction stairs are located. Pass this to warrior.walk_ "
          "to walk in that direction."
          "'play_turn' method.")
level.time_bonus(20)
level.ace_score(19)
level.size(6, 4)
level.stairs(2, 3)

def add_abilities(warrior):
    warrior.add_abilities('walk_')
    warrior.add_abilities('feel')
    warrior.add_abilities('direction_of_stairs')

level.warrior(0, 1, 'east', func=add_abilities)



import glob
import sys

from .base import UnitBase
from .golem import Golem


class Warrior(UnitBase):

    def __init__(self, level=None):
        super(Warrior, self).__init__()
        self.level = level
        self.player_attr = None
        self.name_attr = None
        self.score = 0
        self.golem_abilities = []
        self.max_health = 20

    def play_turn(self, turn):
        return self.player().play_turn(turn)

    def player(self):
        if self.level.player_path() not in sys.path:
            sys.path.insert(0, self.level.player_path())
        if glob.glob(self.level.player_path() + '/player.py'):
            import player

        if self.player_attr:
            return self.player_attr
        else:
            self.player_attr = player.Player()
            return self.player_attr

    def earn_points(self, points):
        self.score = self.score + points
        print ("earns %d points" % points)

    @property
    def attack_power(self):
        return 5

    @property
    def shoot_power(self):
        return 3

    def name(self):
        if self.name_attr:
            return self.name_attr
        else:
            return 'Warrior'

    def __repr__(self):
        return self.name()

    @property
    def character(self):
        return "@"

    def perform_turn(self):
        if self.current_turn.action is None:
            self.say('does nothing')
        super(Warrior, self).perform_turn()

    def add_golem_abilities(self, *abilities):
        self.golem_abilities += abilities

    def has_golem(self):
        if self.golem_abilities:
            return True
        else:
            return False

    def base_golem(self):
        golem = Golem()
        golem.add_abilities(*self.golem_abilities)
        return golem



from .base import AbilityBase


class Walk(AbilityBase):
    def perform(self, direction="forward"):
        self.verify_direction(direction)
        if self._unit.position:
            self._unit.say("walks %s" % direction)
            if self.space(direction).is_empty():
                self._unit.position.move(*self.offset(direction))
            else:
                self._unit.say("bumps into %s" % self.space(direction))

    def description(self):
        return "Move in the given direction (forward by default)."



#  ---
# |>s |
# |s@s|
# | C |
#  ---

level.description("You feel slime on all sides, you're surrounded!")
level.tip("Call warrior.bind_(direction) to bind an enemy to keep him "
          "from attacking. Bound enemies look like captives.")
level.clue("Count the number of enemies around you. Bind an enemy if "
           "there are two or more.")
level.time_bonus(50)
level.ace_score(101)
level.size(3, 3)
level.stairs(0, 0)

def add_abilities(warrior):
    warrior.add_abilities('bind_')
    warrior.add_abilities('rescue_')

level.warrior(1, 1, 'east', func=add_abilities)

level.unit('sludge', 1, 0, 'west')
level.unit('captive', 1, 2, 'west')
level.unit('sludge', 0, 1, 'west')
level.unit('sludge', 2, 1, 'west')



from .base import AbilityBase


class DistanceOf(AbilityBase):
    def description(self):
        return ("Pass a Space as an argument, and it will return an integer "
                "representing the distance to that space.")

    def perform(self, space):
        return self._unit.position.distance_of(space)



import unittest

from pythonwarrior.units.wizard import Wizard


class TestWizard(unittest.TestCase):
    def setUp(self):
        self.wizard = Wizard()

    def test_should_have_look_and_shoot_abilities(self):
        self.assertEqual({'shoot_', 'look'}, set(self.wizard.abilities.keys()))

    def test_should_have_shoot_power_of_11(self):
        self.assertEqual(11, self.wizard.shoot_power)

    def test_should_have_3_max_health(self):
        self.assertEqual(3, self.wizard.max_health)

    def test_should_appear_as_w_on_map(self):
        self.assertEqual('w', self.wizard.character)



"""
 * Returns the size of the screen.
 *
 * @returns {number[]} The size of the screen as an array of [width, height].
"""

import os

def screenSize():
    ts = os.get_terminal_size()

    return [ ts.columns, ts.lines ]

print(screenSize())


import mock
import unittest

from pythonwarrior.abilities.shoot import Shoot


class TestShoot(unittest.TestCase):
    def setUp(self):
        mock_shooter = mock.Mock(shoot_power=2)
        mock_shooter.say.return_value = None
        self.shooter = mock_shooter
        self.shoot = Shoot(self.shooter)

    def test_should_shoot_only_first_unit(self):
        receiver = mock.Mock()
        receiver.is_alive.return_value = True
        receiver.say.return_value = None
        other = mock.Mock()
        self.shoot.multi_unit = mock.Mock()
        self.shoot.multi_unit.return_value = [None, receiver, other, None]

        self.shoot.perform()

        self.shoot.multi_unit.assert_called_once_with('forward', mock.ANY)
        receiver.take_damage.assert_called_once_with(2)
        assert not other.take_damage.called

    def test_should_shoot_and_do_nothing_if_no_units_in_the_way(self):
        self.shoot.multi_unit = mock.Mock(return_value=[None, None])
        self.shoot.perform()

        self.shoot.multi_unit.assert_called_once_with('forward', mock.ANY)



from .base import AbilityBase
import warriorpyUnits as units


class Rescue(AbilityBase):
    def description(self):
        return("Rescue a captive from his chains (earning 20 points)"
               "in given direction (forward by default)")

    def perform(self, direction='forward'):
        self.verify_direction(direction)
        if self.space(direction).is_captive():
            recipient = self.unit(direction)
            self._unit.say("unbinds %(direction)s and rescues %(recipient)s" %
                           {'direction': direction, 'recipient': recipient})
            recipient.unbind()
            if isinstance(recipient, units.captive.Captive):
                recipient.position = None
                self._unit.earn_points(20)
        else:
            self._unit.say("unbinds %s and rescues nothing" % direction)



from .base import AbilityBase


class Listen(AbilityBase):
    def description(self):
        return "Returns an array of all spaces which have units in them."

    def perform(self):
        def _collect_non_warrior_spaces(unit):
            if unit and unit != self._unit:
                return unit.position.space()

        units = map(_collect_non_warrior_spaces,
                    self._unit.position.floor.units)
        units = filter(lambda x: x, units)

        return units



import math
from warriorpyGeography.src.relativeDirections import BACKWARD, FORWARD

from .base import AbilityBase



class Attack(AbilityBase):
    def perform(self, direction="forward"):
        self.verify_direction(direction)
        receiver = self.unit(direction)
        if receiver:
            self._unit.say("attacks %s and hits %s" %
                           (direction, receiver.__class__.__name__))
            if direction == "backward":
                power = math.ceil(self._unit.attack_power/2.0)
            else:
                power = self._unit.attack_power
            self.damage(receiver, power)
        else:
            self._unit.say("attacks %s and hits nothing" % direction)

    def description(self):
        return "Attacks a unit in given direction (forward by default)."



from absoluteDirections import EAST, WEST, SOUTH, NORTH

# /**
#  * Returns the direction of a location from another location (reference
#  * location).
#  *
#  * @param {number[]} location The location as [x, y].
#  * @param {number[]} referenceLocation The reference location as [x, y].
#  *
#  * @returns {string} The direction.
#  */

def getDirectionOfLocation(x1, y1, x2, y2):
    if ( abs(x2 - x1) > abs(y2 - y1) ):
        if x1 > x2:
            return EAST
        
        return WEST

    if y1 > y2:
        return SOUTH

    return NORTH


import mock
import unittest

from pythonwarrior.abilities.rescue import Rescue
from pythonwarrior.units.captive import Captive
from pythonwarrior.units.warrior import Warrior


class TestRescue(unittest.TestCase):
    def setUp(self):
        self.warrior = Warrior()
        self.rescue = Rescue(self.warrior)

    def test_should_rescue_captive(self):
        captive = Captive()
        captive.position = mock.Mock()
        mock_space = mock.Mock()
        mock_space.is_captive.return_value = True
        self.rescue.space = mock.Mock(return_value=mock_space)
        self.rescue.unit = mock.Mock(return_value=captive)
        self.warrior.earn_points = mock.Mock()

        self.rescue.perform()

        self.assertEqual(None, captive.position)
        self.warrior.earn_points.assert_called_once_with(20)



import mock
import unittest
from pythonwarrior.abilities.walk import Walk


class TestWalk(unittest.TestCase):
    def setUp(self):
        self.space = mock.Mock()
        self.space.is_empty.return_value = True
        self.space.unit.return_value = None
        self.position = mock.Mock()
        self.position.relative_space.return_value = self.space
        self.position.move.return_Value = None
        unit = mock.Mock(position=self.position)
        unit.say.return_value = None
        self.walk = Walk(unit)

    def test_should_move_position_forward_when_calling_perform(self):
        self.walk.perform()
        self.position.move.assert_called_once_with(1, 0)

    def test_should_move_position_right_if_that_is_direction(self):
        self.walk.perform("right")
        self.position.move.assert_called_once_with(0, 1)

    def test_should_keep_position_if_something_is_in_the_way(self):
        self.position.move.side_effect = Exception("Boom!")
        self.space.is_empty.return_value = False
        self.walk.perform("right")



from .base import UnitBase


class Sludge(UnitBase):
    def __init__(self):
        super(Sludge, self).__init__()
        self.add_abilities('attack_', 'feel')
        self.max_health = 12

    def __repr__(self):
        return self.name()

    @property
    def attack_power(self):
        return 3

    @property
    def character(self):
        return "s"

    def play_turn(self, turn):
        for direction in ['forward', 'left', 'right', 'backward']:
            if turn.feel(direction).is_player():
                turn.attack_(direction)
                return



import argparse

import warriorpyCore.src.config.Config as Config
import warriorpyCli.src.game.Game as Game


class Runner(object):
    def __init__(self, arguments, stdin, stdout):
        self.arguments = arguments
        self.stdin = stdin
        self.stdout = stdout
        self.game = Game()

    def run(self):
        Config.in_stream = self.stdin
        Config.out_stream = self.stdout
        Config.delay = 0.2
        self.parse_options()
        self.game.start()

    def parse_options(self):
        parser = argparse.ArgumentParser(description='Process options')
        parser.add_argument('-d', '--directory', action=SetDirectory,
                            help='Run under given directory')
        parser.add_argument('-l', '--level', action=SetLevel, type=int,
                            help='Practice level on epic')
        parser.add_argument('-s', '--skip', action=SkipInput, nargs=0,
                            help='Skip user input')
        parser.add_argument('-t', '--time', action=SetDelay, type=float,
                            help='Delay each turn by seconds')

        parser.parse_args()


class SetDirectory(argparse.Action):
    def __call__(self, parser, namespace, values, option_string=None):
        Config.path_prefix = values


class SetLevel(argparse.Action):
    def __call__(self, parser, namespace, values, option_string=None):
        Config.practice_level = values


class SkipInput(argparse.Action):
    def __call__(self, parser, namespace, values, option_string=None):
        Config.skip_input = True


class SetDelay(argparse.Action):
    def __call__(self, parser, namespace, values, option_string=None):
        Config.delay = values



import mock
import unittest

from pythonwarrior.abilities.pivot import Pivot


class TestPivot(unittest.TestCase):
    def setUp(self):
        self.position = mock.Mock()
        mock_unit = mock.Mock(position=self.position)
        self.pivot = Pivot(mock_unit)

    def test_should_flip_around_when_not_passing_arguments(self):
        self.pivot.perform()
        self.position.rotate.assert_called_once_with(2)

    def test_should_rotate_1_when_pivoting_right(self):
        self.pivot.perform('right')
        self.position.rotate.assert_called_once_with(1)



#  --------
# |@      >|
#  --------
level.description("You see before yourself a long hallway with stairs at "
                  "the end. There is nothing in the way.")
level.tip("Call warrior.walk_() to walk forward in the Player "
          "'play_turn' method.")
level.time_bonus(15)
level.ace_score(10)
level.size(8, 1)
level.stairs(7, 0)
def add_abilities(warrior):
    warrior.add_abilities('walk_')
level.warrior(0, 0, 'east', func=add_abilities)



import mock
import unittest

from pythonwarrior.abilities.attack import Attack
from pythonwarrior.units.base import UnitBase


class TestAttack(unittest.TestCase):
    def setUp(self):
        self.attacker = mock.Mock(position=mock.Mock(),
                                  attack_power=3,
                                  say=lambda x: x)
        self.attack = Attack(self.attacker)

    @mock.patch('pythonwarrior.abilities.attack.Attack.unit')
    def test_should_subtract_attack_power_amount_from_health(self, mock_unit):
        receiver = UnitBase()
        receiver.position = mock.Mock()
        receiver._health = 5
        mock_unit.return_value = receiver
        self.attack.perform()
        self.assertEqual(receiver.health, 2)

    def test_should_do_nothing_if_recipient_is_None(self):
        return None



from .base import AbilityBase


class Rest(AbilityBase):
    def description(self):
        return "Gain 10% of max health back, but do nothing more."

    def perform(self):
        if self._unit.health < self._unit.max_health:
            amount = int(self._unit.max_health * 0.1)
            if (self._unit.health + amount) > self._unit.max_health:
                amount = self._unit.max_health - self._unit.health
            self._unit.health += amount
            self._unit.say("receives %(heal_amount)d health from resting, "
                           "up to %(current)d health" %
                           {'heal_amount': amount,
                               'current': self._unit.health})
        else:
            self._unit.say("is already fit as a fiddle")



import glob
import os

import warriorpyCore.config.Config as Config
import warriorpyCore.level.Level as Level
from player_generator import PlayerGenerator
from profile import Profile
import warriorpyCore.tower.Tower as Tower
from ui.ui import UI


class Game(object):
    def __init__(self):
        self._profile = None
        self._current_level = None
        self._next_level = None

    def start(self):
        UI.puts('Welcome to Python Warrior')
        if os.path.exists(Config.path_prefix + '/.profile'):
            self._profile = Profile.load(Config.path_prefix + '/.profile')
        else:
            if not os.path.exists(Config.path_prefix + '/pythonwarrior'):
                self.make_game_directory()

        if self.profile().epic:
            if self.profile().level_after_epic():
                self.go_back_to_normal_mode()
            else:
                self.play_epic_mode()
        else:
            self.play_normal_mode()

    def make_game_directory(self):
        if UI.ask("No pythonwarrior directory found, \
                  would you like to create one?"):
            os.mkdir(Config.path_prefix + '/warriorpy')
        else:
            UI.puts('Unable to continue without directory.')
            raise Exception("Unable to continue without directory.")

    def play_epic_mode(self):
        if Config.delay:
            Config.delay = Config.delay/2

        self.profile().current_epic_score = 0
        self.profile().current_epic_grades = {}
        if Config.practice_level:
            self._current_level = self._next_level = None
            self.profile().level_number = Config.practice_level
            self.play_current_level()
        else:
            playing = True
            while playing:
                self._current_level = self._next_level = None
                self.profile().level_number += 1
                playing = self.play_current_level()
            self.profile().save()

    def play_normal_mode(self):
        if Config.practice_level:
            UI.puts("Unable to practice level while not in epic mode,"
                    "remove -l options")
        else:
            if self.current_level().number == 0:
                self.prepare_next_level()
                UI.puts("First level has been generated."
                        "See the pythonwarrior/%s/README for instructions."
                        % self.profile().directory_name())
            else:
                self.play_current_level()

    def play_current_level(self):
        continue_play = True
        UI.puts("Starting level %d" % self.current_level().number)
        self.current_level().play()
        if self.current_level().is_passed():
            if self.next_level().exists():
                UI.puts("Success! You have found the stairs.")
            else:
                UI.puts("CONGRATULATIONS! You have climbed to the top "
                        "of the tower and rescued the fair maiden Python")
                continue_play = False
            self.current_level().tally_points()
            if self.profile().epic:
                if self.final_report() and not continue_play:
                    UI.puts(self.final_report())
            else:
                self.request_next_level()
        else:
            continue_play = False
            UI.puts("Sorry, you failed level %d. "
                    "Change your script and try again." %
                    self.current_level().number)
            if (not Config.skip_input) and self.current_level().clue \
                    and UI.ask("Would you like to read the "
                               "additional clues for this level?"):
                        UI.puts(self.current_level().clue)
        return continue_play

    def request_next_level(self):
        if not Config.skip_input:
            if self.next_level().exists():
                if UI.ask('Would you like to continue on to the next level?'):
                    self.prepare_next_level()
                    UI.puts('See the updated README in the pythonwarrior/' +
                            self.profile().directory_name() +
                            ' directory')
                else:
                    UI.puts('Staying on current level.'
                            'Try to earn more points next time.')

            else:
                if UI.ask('Would you like to continue on to epic mode?'):
                    self.prepare_epic_mode()
                    UI.puts('Run pythonwarrior again to play epic mode.')
                else:
                    UI.puts("Staying on current level. "
                            "Try to earn more points next time.")

    def prepare_next_level(self):
        self.next_level().load_level()
        PlayerGenerator(self.next_level(), self.current_level()).generate()
        self.profile().level_number += 1
        self.profile().save()

    def prepare_epic_mode(self):
        self.profile().enable_epic_mode()
        self.profile().level_number = 0
        self.profile().save()

    def go_back_to_normal_mode(self):
        self.profile().enable_normal_mode()
        self.prepare_next_level()
        UI.puts('Another level has been added since you started epic, '
                'going back to normal mode')
        UI.puts('See the updated README in the '
                'pythonwarrior/%s directory' % self.profile().directory_name())

    def profiles(self):
        return map(lambda profile: Profile.load(profile), self.profile_paths())

    def profile_paths(self):
        return glob.glob(Config.path_prefix + '/pythonwarrior/**/.profile')

    def profile(self):
        if self._profile is None:
            self._profile = self.choose_profile()
        return self._profile

    def new_profile(self):
        profile = Profile()
        profile.tower_path = UI.choose('tower', self.towers()).path
        profile.warrior_name = UI.request('Enter a name for your warrior: ')
        return profile

    def towers(self):
        return map(lambda path: Tower(path), self.tower_paths())

    def tower_paths(self):
        tower_paths = glob.glob(os.path.normpath(os.path.abspath(__file__) +
                                                 '/../../towers/*'))
        tower_paths = filter(lambda path: os.path.isdir(path), tower_paths)
        return tower_paths

    def current_level(self):
        if not self._current_level:
            self._current_level = self.profile().current_level()
        return self._current_level

    def next_level(self):
        if not self._next_level:
            self._next_level = self.profile().next_level()
        return self._next_level

    def final_report(self):
        average_grade = self.profile().calculate_average_grade()
        if average_grade and not Config.practice_level:
            report = ""
            report += ("Your average grade for this tower is: %s\n\n" %
                       Level.grade_letter(average_grade))

    def choose_profile(self):
        profile = UI.choose('profile',
                            self.profiles() + [['new', 'New Profile']])
        if profile == 'new':
            profile = self.new_profile()
            if filter(lambda prof: prof.player_path == profile.player_path,
                      self.profiles()):
                if UI.ask('Are you sure you want to replace your existing'
                          'profile for this tower?'):
                    UI.puts('Replacing existing profile.')
                    return profile
                else:
                    UI.puts('Not replacing profile.')
                    raise Exception('Not replacing profile')
            else:
                return profile
        else:
            return profile



#  -------
# |@ Sa S>|
#  -------

level.description("You can hear bow strings being stretched.")
level.tip("No new abilities this time, but you must be careful not to rest while taking damage. Save a self.health instance variable and compare it on each turn to see if you're taking damage.")
level.clue("Set self.health to your current health at the end of the turn. If this is greater than your current health next turn then you know you're taking damage and shouldn't rest.")

level.time_bonus(45)
level.ace_score(90)
level.size(7, 1)
level.stairs(6, 0)

level.warrior(0, 0, 'east')

level.unit('thick_sludge', 2, 0, 'west')
level.unit('archer', 3, 0, 'west')
level.unit('thick_sludge', 5, 0, 'west')



#  -----
# |    S|
# |@> SC|
#  -----

level.description("You can feel the stairs right next to you, "
                  "but are you sure you want to go up them right away?")
level.tip("You'll get more points for clearing the level first. "
          "Use warrior.feel().is_stairs() and warrior.feel().is_empty() "
          "to determine where to go.")
level.clue("If going towards a unit is the same direction as the stairs, "
           "try moving another empty direction until you can safely move "
           "toward the enemies.")
level.time_bonus(45)
level.ace_score(107)
level.size(5, 2)
level.stairs(1, 1)

level.warrior(0, 1, 'east')

level.unit('thick_sludge', 4, 0, 'west')
level.unit('thick_sludge', 3, 1, 'north')
level.unit('captive', 4, 1, 'west')



import unittest

from pythonwarrior.units.archer import Archer


class TestArcher(unittest.TestCase):
    def setUp(self):
        self.archer = Archer()

    def test_should_have_look_and_shoot_abilities(self):
        self.assertEqual({'shoot_', 'look'}, set(self.archer.abilities.keys()))

    def test_should_have_shoot_power_of_3(self):
        self.assertEqual(3, self.archer.shoot_power)

    def test_should_have_7_max_health(self):
        self.assertEqual(7, self.archer.max_health)

    def test_should_appear_as_a_on_map(self):
        self.assertEqual('a', self.archer.character)



import mock
import unittest
from pythonwarrior.abilities.feel import Feel


class TestFeel(unittest.TestCase):
    def setUp(self):
        self.unit = mock.Mock()
        self.unit.say.return_value = None
        self.feel = Feel(self.unit)

    def test_should_get_object_at_position_from_offset(self):
        self.unit.position.relative_space.return_value = [1, 0]
        self.feel.perform('forward')
        self.unit.position.relative_space.assert_called_once_with(1, 0)



from .sludge import Sludge


class ThickSludge(Sludge):
    def __init__(self):
        super(ThickSludge, self).__init__()
        self.max_health = 24

    @property
    def character(self):
        return "S"



import warriorpyGeography.src.absoluteDirections.ABSOLUTE_DIRECTIONS as ABSOLUTE_DIRECTIONS
import warriorpyGeography.src.relativeDirections.RELATIVE_DIRECTIONS as RELATIVE_DIRECTIONS

class Position(object):
    DIRECTIONS = ABSOLUTE_DIRECTIONS
    RELATIVE_DIRECTIONS = RELATIVE_DIRECTIONS

    def __init__(self, floor, x, y, direction=None):
        self.floor = floor
        self.x = x
        self.y = y
        self.direction_index = self.DIRECTIONS.index(direction or "north")

    def at(self, x, y):
        return self.x == x and self.y == y

    @property
    def direction(self):
        return self.DIRECTIONS[self.direction_index]

    def rotate(self, amount):
        self.direction_index = (self.direction_index + amount) % 4

    def relative_space(self, forward, right=None):
        if right is None:
            right = 0
        return self.floor.space(*(self.translate_offset(forward, right)))

    def space(self):
        return self.floor.space(self.x, self.y)

    def move(self, forward, right=None):
        if right is None:
            right = 0
        (x, y) = self.translate_offset(forward, right)
        self.x = x
        self.y = y

    def distance_from_stairs(self):
        return self.distance_of(self.floor.stairs_space())

    def distance_of(self, space):
        (x, y) = space.location()
        return abs(self.x - x) + abs(self.y - y)

    def relative_direction_of_stairs(self):
        return self.relative_direction_of(self.floor.stairs_space())

    def relative_direction_of(self, space):
        return self.relative_direction(self.direction_of(space))

    def direction_of(self, space):
        (space_x, space_y) = space.location()
        if abs(self.x - space_x) > abs(self.y - space_y):
            return 'east' if space_x > self.x else 'west'
        else:
            return 'south' if space_y > self.y else 'north'

    def relative_direction(self, direction):
        offset = (self.DIRECTIONS.index(direction) - self.direction_index) % 4
        return self.RELATIVE_DIRECTIONS[offset]

    def translate_offset(self, forward, right):
        if self.direction == 'north':
            return [self.x + right, self.y - forward]
        if self.direction == 'east':
            return [self.x + forward, self.y + right]
        if self.direction == 'south':
            return [self.x - right, self.y + forward]
        if self.direction == 'west':
            return [self.x - forward, self.y - right]



from .base import AbilityBase


class Pivot(AbilityBase):
    ROTATION_DIRECTIONS = ['forward', 'right', 'backward', 'left']

    def description(self):
        return "Rotate 'left', 'right', or 'backward' (default)"

    def perform(self, direction='backward'):
        self.verify_direction(direction)
        self._unit.position.rotate(self.ROTATION_DIRECTIONS.index(direction))
        self._unit.say("pivots %s" % direction)



import unittest
from pythonwarrior.units.captive import Captive


class TestCaptive(unittest.TestCase):
    def setUp(self):
        self.captive = Captive()

    def test_should_have_1_max_health(self):
        self.assertEqual(self.captive.max_health, 1)

    def test_should_appear_as_C_on_map(self):
        self.assertEqual(self.captive.character, "C")

    def test_should_be_bound_by_default(self):
        self.assertTrue(self.captive.is_bound())

    def test_should_not_have_explode_ability_by_default(self):
        self.assertNotIn('explode_', self.captive.abilities)



import mock
import unittest
from pythonwarrior.level import Level
from pythonwarrior.level_loader import LevelLoader
from pythonwarrior.profile import Profile


class TestLevelLoader(unittest.TestCase):
    def setUp(self):
        self.profile = Profile()
        self.level = Level(self.profile, 1)
        self.loader = LevelLoader(self.level)

    def test_should_be_able_to_add_description_tip_and_clue(self):
        self.loader.description('foo')
        self.loader.tip('bar')
        self.loader.clue('clue')
        self.assertEqual(self.level.description, 'foo')
        self.assertEqual(self.level.tip, 'bar')
        self.assertEqual(self.level.clue, 'clue')

    def test_should_be_able_to_set_size(self):
        self.loader.size(5, 3)
        self.assertEqual(self.level.floor.width, 5)

    def test_should_be_able_to_add_stairs(self):
        with mock.patch.object(self.level.floor, 'place_stairs') as mock_ps:
            self.loader.stairs(1, 2)
            mock_ps.assert_called_once_with(1, 2)

    def test_should_accept_a_function_to_apply_to_new_unit(self):
        def some_function(unit):
            unit._health = 20
            unit.take_damage(15)

        unit = self.loader.unit('unit_base', 1, 2, "north", func=some_function)
        self.assertEqual(unit.__class__.__name__, "UnitBase")
        self.assertTrue(unit.position.at(1, 2))
        self.assertEqual(unit.health, 5)

    @unittest.skip
    def test_should_be_able_to_add_multi_word_units(self):
        self.loader.unit('thick_sludge')

    def test_should_build_warrior_from_profile(self):
        def some_function(unit):
            unit.take_damage(1)
        self.loader.warrior(1, 2, func=some_function)

    def test_should_be_able_to_set_time_bonus(self):
        self.loader.time_bonus(100)
        self.assertEqual(self.level.time_bonus, 100)

    def test_should_be_able_to_set_ace_score(self):
        self.loader.ace_score(100)
        self.assertEqual(self.level.ace_score, 100)



import mock
import os
import unittest

from pythonwarrior.floor import Floor
from pythonwarrior.level import Level
from pythonwarrior.profile import Profile
from pythonwarrior.units.warrior import Warrior


class TestLevel(unittest.TestCase):
    def setUp(self):
        self.profile = Profile()
        self.floor = Floor()
        self.level = Level(self.profile, 1)
        self.level.floor = self.floor

    def test_should_consider_passed_When_warrior_is_on_stairs(self):
        self.level.warrior = Warrior()
        self.floor.add(self.level.warrior, 0, 0, 'north')
        self.floor.place_stairs(0, 0)
        self.assertTrue(self.level.is_passed())

    def test_should_default_time_bonus_to_zero(self):
        self.assertEqual(self.level.time_bonus, 0)

    def test_should_have_a_grade_relative_to_ace_score(self):
        self.level.ace_score = 100
        self.assertEqual(self.level.grade_for(110), "S")
        self.assertEqual(self.level.grade_for(100), "S")
        self.assertEqual(self.level.grade_for(99), "A")
        self.assertEqual(self.level.grade_for(89), "B")
        self.assertEqual(self.level.grade_for(79), "C")
        self.assertEqual(self.level.grade_for(69), "D")
        self.assertEqual(self.level.grade_for(59), "F")

    def test_should_have_no_grade_if_there_is_no_ace_score(self):
        self.assertIsNone(self.level.ace_score)
        self.assertIsNone(self.level.grade_for(100))

    def test_should_load_file_contents_into_level(self):
        with mock.patch.object(self.level, 'load_path',
                               return_value='path/to/level.py'):
            mock_file = mock.Mock(
                read=mock.Mock(return_value="level.description('foo')"))
            with mock.patch('__builtin__.open',
                            return_value=mock_file):
                self.level.load_level()
                self.assertEqual(self.level.description, 'foo')

    def test_should_have_a_player_path_from_profile(self):
        with mock.patch.object(self.profile, '_player_path', 'path/to/player'):
            self.assertEqual(self.level.player_path(), 'path/to/player')

    def test_should_have_load_path_from_profile_tower_with_level_number(self):
        with mock.patch.object(self.profile, 'tower_path', 'path/to/tower'):
            self.assertEqual(self.level.load_path(),
                             os.path.abspath(
                                 'pythonwarrior/towers/tower/level_001.py'))

    @mock.patch('pythonwarrior.level.os.path.exists')
    def test_should_exist_if_file_exists(self, mock_exists):
        self.level.load_path = mock.Mock(return_value='/foo/bar')
        mock_exists.return_value = True
        self.assertTrue(self.level.exists())
        mock_exists.assert_called_once_with('/foo/bar')

    @mock.patch('pythonwarrior.level.PlayerGenerator')
    def test_should_generate_player_files(self, mock_pg):
        generator = mock.Mock()
        mock_pg.return_value = generator
        self.level.load_level = mock.Mock()
        self.level.generate_player_files()
        generator.generate.assert_called_once_with()
        self.level.load_level.assert_called_once_with()

    def test_should_setup_warrior_with_profile_abilities(self):
        self.profile.abilities = ['foo', 'bar']
        warrior = mock.Mock()
        self.level.setup_warrior(warrior)
        warrior.add_abilities.assert_called_once_with('foo', 'bar')

    def test_should_setup_warrior_with_profile_name(self):
        self.profile.warrior_name = "Joe"
        warrior = mock.Mock()
        self.level.setup_warrior(warrior)
        self.assertEqual(warrior.name_attr, "Joe")


class TestPlaying(unittest.TestCase):
    def setUp(self):
        self.profile = Profile()
        self.floor = Floor()
        self.level = Level(self.profile, 1)
        self.level.floor = self.floor
        self.level.load_level = mock.Mock()
        self.level.is_failed = mock.Mock(return_value=False)

    def test_load_level_once_when_playing_multiple_turns(self):
        self.level.play(2)
        self.level.load_level.assert_called_once_with()

    def test_prepare_turn_and_play_turn_each_object_specified_amount(self):
        unit = Warrior()
        unit.prepare_turn = mock.Mock()
        unit.perform_turn = mock.Mock()
        self.floor.add(unit, 0, 0, 'north')
        self.level.play(2)
        unit.prepare_turn.assert_called_with()
        unit.perform_turn.assert_called_with()
        self.assertEqual(2, unit.prepare_turn.call_count)
        self.assertEqual(2, unit.perform_turn.call_count)

    def test_should_return_immediately_when_passed(self):
        unit = Warrior()
        unit.turn = mock.Mock()
        self.floor.add(unit, 0, 0, 'north')
        self.level.is_passed = mock.Mock(return_value=True)
        self.level.play(2)

    def test_call_fxn_in_play_for_each_turn(self):
        self.i = 0

        def foo():
            self.i += 1

        self.level.play(2, foo)
        self.assertEqual(self.i, 2)

    def test_should_count_down_time_bonus_once_each_turn(self):
        self.level.time_bonus = 10
        self.level.play(3)
        self.assertEqual(self.level.time_bonus, 7)

    def test_should_count_down_time_bonus_to_zero(self):
        self.level.time_bonus = 2
        self.level.play(5)
        self.assertEqual(self.level.time_bonus, 0)

    def test_should_have_a_pretty_score_calculation(self):
        self.assertEqual(self.level.score_calculation(123, 45),
                         "123 + 45 = 168")

    def test_should_not_have_score_calc_when_starting_score_is_zero(self):
        self.assertEqual(self.level.score_calculation(0, 45), "45")


class TestTallying(unittest.TestCase):
    def setUp(self):
        self.profile = Profile()
        self.floor = Floor()
        self.level = Level(self.profile, 1)
        self.level.floor = self.floor
        self.level.load_level = mock.Mock()
        self.level.is_failed = mock.Mock(return_value=False)
        self.level.floor.other_units = mock.Mock(return_value=[mock.Mock()])
        self.warrior = mock.Mock(score=0, abilities={})
        self.level.warrior = self.warrior

    def test_should_add_warrior_score_to_profile(self):
        self.warrior.score = 30
        self.profile.score = 0
        self.level.tally_points()
        self.assertEqual(self.profile.score, 30)

    def test_should_apply_warrior_abilities_to_profile(self):
        self.warrior.abilities = {'foo': None, 'bar': None}
        self.level.tally_points()
        self.assertEqual(['foo', 'bar'], self.profile.abilities)

    def test_should_apply_time_bonus_to_profile_score(self):
        self.level.time_bonus = 20
        self.level.tally_points()
        self.assertEqual(20, self.profile.score)

    def test_should_give_20_percent_bonus_when_no_other_units_left(self):
        self.level.floor.other_units = mock.Mock(return_value=[])
        self.warrior.score = 10
        self.level.time_bonus = 10
        self.level.tally_points()
        self.assertEqual(24, self.profile.score)



from setuptools import setup, find_packages


setup(
    name='Warrior-pytho-2',
    version='0.0.1',
    packages=['warriorpy', 'warriorpy.towers.beginner', 'warriorpy.towers.intermediate','warriorpy.warriorpyAbilities.src', 'warriorpy.warriorpyCli.src', 'warriorpy.warriorpyCli.src.ui', 'warriorpy.warriorpyCli.templates' ,'warriorpy.warriorpyCore.src', 'warriorpy.warriorpyGeography.src', 'warriorpy.warriorpyUnits.src'],
    packages_dir={'warriorpy': 'warriorpy'},
    scripts=['bin/warriorpy.py'],
    author="Divyansh Dwivedi",
    author_email="justdvnsh2208@gmail.com",
    description=("An exciting game of programming and Artificial Intelligence"),
    long_description=open('README.md').read(),
    install_requires=['jinja2'],
    license="MIT",
    url="https://github.com/justdvnsh/warriorpy",
    keywords="python ruby javascript rubywarrior warriorjs warriorpy",
    classifiers=[
        "Programming Language :: Python",
        "License :: OSI Approved :: MIT License",
        "Development Status :: 3 - Alpha"
    ]
)



#  ---------
# |@ s ss s>|
#  ---------

level.description("The air feels thicker than before. There must be a horde of sludge.")
level.tip("Be careful not to die! Use warrior.health to keep an eye on your health, and warrior.rest! to earn 10% of max health back.")
level.clue("When there is no enemy ahead of you call warrior.rest_() until health is full before walking forward.")
level.time_bonus(35)
level.ace_score(71)
level.size(9, 1)
level.stairs(8, 0)
def add_abilities(warrior):
    warrior.add_abilities('health')
    warrior.add_abilities('rest_')
level.warrior(0, 0, 'east', func=add_abilities)

level.unit('sludge', 2, 0, 'west')
level.unit('sludge', 4, 0, 'west')
level.unit('sludge', 5, 0, 'west')
level.unit('sludge', 7, 0, 'west')



import mock
import unittest
from pythonwarrior.units.warrior import Warrior


class TestWarrior(unittest.TestCase):
    def setUp(self):
        self.warrior = Warrior()

    def test_should_default_name_to_warrior(self):
        self.assertEqual(self.warrior.name(), 'Warrior')

    def test_should_be_able_to_set_name(self):
        self.warrior.name_attr = "Joe"
        self.assertEqual(self.warrior.name(), "Joe")
        self.assertEqual(str(self.warrior), "Joe")

    def test_should_have_20_max_health(self):
        self.assertEqual(self.warrior.max_health, 20)

    def test_should_have_0_score_at_beginning_and_be_able_to_earn_points(self):
        self.assertEqual(self.warrior.score, 0)
        self.warrior.earn_points(5)
        self.assertEqual(self.warrior.score, 5)

    @mock.patch('pythonwarrior.units.warrior.Warrior.player')
    def test_should_call_player_play_turn_and_pass_turn_to_player(self,
                                                                  mock_player):
        returned_player = mock.Mock()
        mock_player.return_value = returned_player
        self.warrior.play_turn('turn')
        mock_player.assert_called_once_with()
        returned_player.play_turn.assert_called_once_with('turn')

    @unittest.skip
    @mock.patch('pythonwarrior.units.warrior.player.Player')
    def test_should_instantiate_player_first_time_and_return_same_object_next_time(self, mock_player):
        player = mock.Mock()
        mock_player.return_value = player
        self.assertEqual(self.warrior.player(), player)
        self.assertEqual(self.warrior.player(), player)

    def test_should_have_an_attack_power_of_5(self):
        self.assertEqual(self.warrior.attack_power, 5)

    def test_should_have_a_shoot_power_of_3(self):
        self.assertEqual(self.warrior.shoot_power, 3)

    def test_should_appear_as_AT_symbol_on_map(self):
        self.assertEqual(self.warrior.character, "@")

    def test_should_be_able_to_add_golem_abilities_which_are_used_on_base_golem(self):
        self.warrior.add_golem_abilities("walk")
        self.assertEqual(self.warrior.base_golem().abilities.keys(), ["walk"])



from .base import AbilityBase


class Bind(AbilityBase):
    def description(self):
        return("Binds a unit in given direction to keep him "
               "from moving (forward by default).")

    def perform(self, direction='forward'):
        self.verify_direction(direction)
        receiver = self.unit(direction)
        if receiver:
            self._unit.say("binds %(direction)s and restricts "
                           "%(receiver)s" %
                           {'direction': direction, 'receiver': receiver})
            receiver.bind()
        else:
            self._unit.say("binds %s and restricts nothing" % direction)



class Config(object):
    delay = None
    in_stream = None
    out_stream = None
    practice_level = None
    path_prefix = "."
    skip_input = None

    @staticmethod
    def reset():
        Config.delay = None
        Config.in_stream = None
        Config.out_stream = None
        Config.practice_level = None
        Config.path_prefix = "."
        Config.skip_input = None



from .base import AbilityBase


class DirectionOf(AbilityBase):
    def description(self):
        return ("Pass a Space as an argument, and the direction "
                "(:left, :right, :forward, :backward) "
                "to that space will be returned.")

    def perform(self, space):
        return self._unit.position.relative_direction_of(space)



#  ------
# |>a S @|
#  ------

level.description("You feel a wall right in front of you and an opening behind you.")
level.tip("You are not as effective at attacking backward. Use warrior.feel().is_wall() and warrior.pivot_() to turn around.")

level.time_bonus(30)
level.ace_score(50)
level.size(6, 1)
level.stairs(0, 0)

def add_abilities(warrior):
    warrior.add_abilities('pivot_')
level.warrior(5, 0, 'east', func=add_abilities)

level.unit('archer', 1, 0, 'east')
level.unit('thick_sludge', 3, 0, 'east')



from .base import AbilityBase


class Shoot(AbilityBase):
    def description(self):
        return("Shoot your bow & arrows in given direction"
               "(forward by default)")

    def perform(self, direction='forward'):
        self.verify_direction(direction)
        units = self.multi_unit(direction, range(1, 4))
        units = filter(lambda unit: unit, units)
        if len(units):
            receiver = units
            self._unit.say("shoots %(direction)s and hits %(receiver)s" %
                           {'direction': direction, 'receiver': receiver})
            self.damage(receiver, self._unit.shoot_power)
        else:
            self._unit.say("shoots and hits nothing")

    def multi_unit(self, direction, range_list):
        return map(lambda n: self.unit(direction, n), range_list)



import unittest
from pythonwarrior.units.base import UnitBase
from pythonwarrior.floor import Floor


class TestPosition(unittest.TestCase):
    def setUp(self):
        self.unit = UnitBase()
        self.floor = Floor()
        self.floor.width = 6
        self.floor.height = 5
        self.floor.add(self.unit, 1, 2, "north")
        self.position = self.unit.position

    def test_at_should_be_true_if_position_matches_x_and_y(self):
        self.assertTrue(self.position.at(1, 2))

    def test_should_rotate_clockwise(self):
        self.assertEqual('north', self.position.direction)
        for direction in ['east', 'south', 'west', 'north', 'east']:
            self.position.rotate(1)
            self.assertEqual(direction, self.position.direction)

    def test_should_rotate_counter_clockwise(self):
        self.assertEqual('north', self.position.direction)
        for direction in ['west', 'south', 'east', 'north', 'west']:
            self.position.rotate(-1)
            self.assertEqual(direction, self.position.direction)

    def test_should_get_relative_space_in_front(self):
        unit = UnitBase()
        self.floor.add(unit, 1, 1)
        self.assertFalse(self.position.relative_space(1).is_empty())

    def test_should_get_relative_object_in_front_when_rotated(self):
        unit = UnitBase()
        self.floor.add(unit, 2, 2)
        self.position.rotate(1)
        self.assertFalse(self.position.relative_space(1).is_empty())

    def test_should_get_relative_object_diagonally(self):
        unit = UnitBase()
        self.floor.add(unit, 0, 1)
        self.assertFalse(self.position.relative_space(1, -1).is_empty())

    def test_should_get_relative_object_diagonally_when_rotating(self):
        unit = UnitBase()
        self.floor.add(unit, 0, 1)
        self.position.rotate(2)
        self.assertFalse(self.position.relative_space(-1, 1).is_empty())

    def test_should_move_object_on_floor_relatively(self):
        self.assertEqual(self.unit, self.floor.get(1, 2))
        self.position.move(-1, 2)
        self.assertEqual(None, self.floor.get(1, 2))
        self.assertEqual(self.unit, self.floor.get(3, 3))
        self.position.rotate(1)
        self.position.move(-1)
        self.assertEqual(None, self.floor.get(3, 3))
        self.assertEqual(self.unit, self.floor.get(2, 3))

    def test_should_return_distance_from_stairs_as_0_when_on_stairs(self):
        self.floor.place_stairs(1, 2)
        self.assertEqual(0, self.position.distance_from_stairs())

    def test_should_return_distance_from_stairs_in_both_directions(self):
        self.floor.place_stairs(0, 3)
        self.assertEqual(2, self.position.distance_from_stairs())

    def test_should_return_relative_direction_of_stairs(self):
        self.floor.place_stairs(0, 0)
        self.assertEqual('forward',
                         self.position.relative_direction_of_stairs())

    def test_should_return_relative_direction_of_given_space(self):
        result = self.position.relative_direction_of(self.floor.space(5, 3))
        self.assertEqual('right', result)

    def test_should_be_able_to_determine_relative_direction(self):
        self.assertEqual('forward', self.position.relative_direction('north'))
        self.assertEqual('backward', self.position.relative_direction('south'))
        self.assertEqual('left', self.position.relative_direction('west'))
        self.assertEqual('right', self.position.relative_direction('east'))
        self.position.rotate(1)
        self.assertEqual('left', self.position.relative_direction('north'))
        self.position.rotate(1)
        self.assertEqual('backward', self.position.relative_direction('north'))
        self.position.rotate(1)
        self.assertEqual('right', self.position.relative_direction('north'))

    def test_should_return_a_space_at_the_same_location_as_position(self):
        self.assertEqual([1, 2], self.position.space().location())

    def test_should_return_distance_of_given_space(self):
        self.assertEqual(5, self.position.distance_of(self.floor.space(5, 3)))
        self.assertEqual(3, self.position.distance_of(self.floor.space(4, 2)))



import shutil
import os
from jinja2 import Environment, PackageLoader


class PlayerGenerator(object):
    def __init__(self, level, previous_level):
        self.level = level
        self.previous_level = previous_level

    def generate(self):
        if self.level.number == 1:
            if not os.path.exists(self.level.player_path()):
                os.makedirs(self.level.player_path())
            shutil.copy(self.templates_path() +
                        '/player.py', self.level.player_path() + '/player.py')

        env = Environment(loader=PackageLoader('warriorpy', 'templates'))
        readme_template = env.get_template('README')

        readme = open(self.level.player_path() + '/README', 'w')
        readme.write(readme_template.render(level=self.level))
        readme.close()

    def templates_path(self):
        return os.path.normpath(os.path.dirname(__file__) + "/templates")



import unittest
import mock
from pythonwarrior.abilities.base import AbilityBase


class TestAbilityBase(unittest.TestCase):
    def setUp(self):
        unit = mock.Mock()
        self.ability = AbilityBase(unit)

    def test_should_have_offset_for_directions(self):
        self.assertEqual(self.ability.offset('forward'), [1, 0])
        self.assertEqual(self.ability.offset('right'), [0, 1])
        self.assertEqual(self.ability.offset('backward'), [-1, 0])
        self.assertEqual(self.ability.offset('left'), [0, -1])

    def test_should_have_offset_for_relative_forward_and_right_amounts(self):
        self.assertEqual(self.ability.offset('forward',
                                             forward=2), [2, 0])
        self.assertEqual(self.ability.offset('forward',
                                             forward=2, right=1), [2, -1])
        self.assertEqual(self.ability.offset('right',
                                             forward=2, right=1), [1, 2])
        self.assertEqual(self.ability.offset('backward',
                                             forward=2, right=1), [-2, 1])
        self.assertEqual(self.ability.offset('left',
                                             forward=2, right=1), [-1, -2])

    @mock.patch('pythonwarrior.abilities.base.AbilityBase.space')
    def test_should_fetch_unit_at_given_direction_with_distance(self,
                                                                mock_space):
        space = mock.Mock()
        space.unit = 'unit'
        mock_space.return_value = space
        self.assertEqual(self.ability.unit('right', 3, 1), 'unit')

    def test_should_have_no_description(self):
        self.assertEqual(self.ability.description(), None)

    def test_should_raise_an_exception_if_direction_isnt_recognized(self):
        self.assertRaises(Exception, self.ability.verify_direction, 'foo')

if __name__ == '__main__':
    unittest.main()



from .base import AbilityBase


class Feel(AbilityBase):
    def description(self):
        return "Returns a Space for the given direction (forward by default)"

    def perform(self, direction='forward'):
        if direction is None:
            direction = 'forward'
        self.verify_direction(direction)
        return self.space(direction)



import mock
import unittest

from pythonwarrior.abilities.direction_of_stairs import DirectionOfStairs


class TestDirectionOfStairs(unittest.TestCase):
    def setUp(self):
        self.position = mock.Mock()
        self.direction = DirectionOfStairs(mock.Mock(position=self.position))

    def test_should_return_relative_direction_of_stairs(self):
        self.position.relative_direction_of_stairs.return_value = 'left'
        self.assertEqual('left', self.direction.perform())
